@@ -26,6 +26,10 @@
 /* The old route entry structure in newer BSDs */
 #undef HAVE_ORTENTRY
 
+/* Do we ahve any route entry structure ? */
+#undef HAVE_RTENTRY
+
+
 /* Whether to use the local REGEX functions */
 #undef REGEX_MALLOC
 
@@ -59,6 +63,7 @@
 #undef OPENBSD
 #undef HAVE_SYS_ACL_H
 #undef NOTBROKEN
+#undef NT
 
 /* SVR4 header stuff */
 
