@@ -104,19 +104,20 @@ else
    }
 }
 
-
 /**************************************************************/
 
 /* Prints a message digest in hexadecimal */
 
-void cfMDPrint (digest)
+char *cfMDPrint (digest)
 unsigned char digest[16];
 
 { unsigned int i;
-  static buffer[36]; 
+  static char buffer[36]; 
 
 for (i = 0; i < 16; i++)
    {
    sprintf((char *)(buffer+2*i),"%02x", digest[i]);
    }
+
+return buffer; 
 }    
