@@ -213,9 +213,13 @@ extern int errno;
 char *strdup();
 #endif
 
-
-#include "../pub/regex.h"
-
+#ifdef HAVE_RXPOSIX_H
+# include <rxposix.h>
+#elif  HAVE_REGEX_H
+# include <regex.h>
+#else
+# include "../pub/regex.h"
+#endif
 
 /*******************************************************************/
 /* Various defines                                                 */
@@ -224,11 +228,11 @@ char *strdup();
 #define true  1
 #define false 0
 #define bufsize 4096
-#define buffer_margin 10
+#define buffer_margin 32
 #define maxvarsize 128
 #define maxlinksize 256
 #define maxlinklevel 4
-#define maxargs 30
+#define maxargs 31
 #define hashtablesize 119   /* prime number */
 #define macroalphabet 60    /* a-z, A-Z plus a bit */
 #define maxshellargs 10
@@ -238,7 +242,7 @@ char *strdup();
 #define extra_space 8      /* pads items during AppendItem for eol handling in editfiles */
 #define cfinfinity (int)999999999
 #define ticksperday 86400 /* 60 * 60 *24 */
-
+#define cf_not_connected -1
 
 /* Need this to to avoid conflict with solaris 2.6 and db.h */
 
@@ -272,7 +276,7 @@ char *strdup();
 #define CF_SAVED ".cfsaved"
 #define CF_EDITED ".cfedited"
 #define CF_NEW ".cfnew"
-#define CFD_TERMINATOR "---cfengine"
+#define CFD_TERMINATOR "---cfXen/gine/cfXen/gine---"
 #define CFD_TRUE "CFD_TRUE"
 #define CFD_FALSE "CFD_FALSE"
 #define CF_NOUSER -99
@@ -561,6 +565,9 @@ enum fileattr
    cfdelfstab,
    cfstealth,
    cfchksum,
+   cfflags,
+   cfsecure,
+   cfroot,
    cfbad
    };
 
@@ -627,6 +634,7 @@ enum vnames
    cfifelapsed,
    cfextension,
    cfsuspicious,
+   cfdefcopy,
    nonexistentvar
    };
 
@@ -996,6 +1004,8 @@ struct File
    char   log;
    char   inform;
    char   checksum;   /* m=md5 n=none */
+   u_long plus_flags;    /* for *BSD chflags */
+   u_long minus_flags;    /* for *BSD chflags */
    };
 
 /*******************************************************************/
@@ -1058,20 +1068,23 @@ struct Image
 
    struct cfstat *cache;                              /* stat cache */
    struct CompressedArray *inode_cache;              /* inode cache */
-
+ 
    struct in_addr *dns;                      /* Cache gethostbyname */
    
    struct Image *next;
    struct Item *acl_aliases;
    char   log;
    char   inform;
+   short  secure;
+   u_long plus_flags;    /* for *BSD chflags */
+   u_long minus_flags;    /* for *BSD chflags */      
    };
 
 /*******************************************************************/
 
 struct UidList
    {
-   int uid;
+   uid_t uid;
    struct UidList *next;
    };
 
@@ -1081,7 +1094,7 @@ struct UidList *MakeUidList();
 
 struct GidList
    {
-   int gid;
+   gid_t gid;
    struct GidList *next;
    };
 
@@ -1179,6 +1192,7 @@ struct ShellComm
    struct ShellComm  *next;
    char              log;
    char              inform;
+   char              *defines;
    };
 
 /*******************************************************************/
@@ -1187,6 +1201,8 @@ struct Auth
    {
    char *path;
    struct Item *accesslist;
+   struct Item *maproot;    /* which hosts should have root read access */
+   int secure;              /* which files HAVE to be transmitted securely */
    struct Auth *next;
    };
 
@@ -1259,6 +1275,40 @@ struct Auth
 #define S_IXOTH 00001
 #endif
 
+/********************************************************************/
+/* *BSD chflags stuff - Andreas.Klussmann@infosys.heitec.net        */
+/********************************************************************/
+
+# if !defined UF_NODUMP
+#  define UF_NODUMP 0
+# endif
+# if !defined UF_IMMUTABLE
+#  define UF_IMMUTABLE 0
+# endif
+# if !defined UF_APPEND
+#  define UF_APPEND 0
+# endif
+# if !defined UF_OPAQUE
+#  define UF_OPAQUE 0
+# endif
+# if !defined UF_NOUNLINK
+#  define UF_NOUNLINK 0
+# endif
+# if !defined SF_ARCHIVED
+#  define SF_ARCHIVED 0
+# endif
+# if !defined SF_IMMUTABLE
+#  define SF_IMMUTABLE 0
+# endif
+# if !defined SF_APPEND
+#  define SF_APPEND 0
+# endif
+# if !defined SF_NOUNLINK
+#  define SF_NOUNLINK 0
+# endif
+# define CHFLAGS_MASK  ( UF_NODUMP | UF_IMMUTABLE | UF_APPEND | UF_OPAQUE | UF_NOUNLINK | SF_ARCHIVED | SF_IMMUTABLE | SF_APPEND | SF_NOUNLINK )
+
+
 /********************************************************************/
 /* Misc. prototypes                                                 */
 /********************************************************************/
@@ -1267,6 +1317,7 @@ char *GetMacroValue();
 char *Get2DListEnt();
 char *CanonifyName();
 char *ASUniqueName();
+char *cfMDPrint ();
 
 char ToUpper();
 char ToLower();
@@ -1276,4 +1327,4 @@ char *ToLowerStr();
 char *ReadLastNode();
 char *AbsLinkPath();
 
-FILE *cfpopen();
+FILE *cfpopen(), *cfpopen_sh();
