@@ -34,18 +34,23 @@ extern FILE *yyin;
 #define connection 1
 #define CFD_INPUT "cfd.conf"
 #define CFD_SOURCE 1
+#define RFC931_PORT 113
 
 /**********************************************************************/
 
 char *COMMANDS[] =
    {
    "EXEC",
-   "AUTH",
+   "AUTH",  /* old protocol */
    "GET",
    "OPENDIR",
    "SYNCH",
    "CLASSES",
    "MD5",
+   "SMD5",
+   "CAUTH",
+   "SSYNCH",
+   "SGET",
    NULL
    };
 
@@ -58,6 +63,10 @@ enum COMMS
    cfd_synch,
    cfd_classes,
    cfd_md5,
+   cfd_smd5,
+   cfd_cauth,
+   cfd_ssynch,
+   cfd_sget,
    cfd_bad
    };
 
@@ -65,14 +74,20 @@ struct cfd_connection
    {
    int id_verified;
    int synchronized;
+   int maproot;
    int sd_reply;
    char hostname[maxvarsize];
    char username[maxvarsize];
+   char signature[8];
+   char ipaddr[20];
    };
 
-struct cfd_thread_arg
+struct cfd_get_arg
    {
    struct cfd_connection *connect;
+   int secure;
+   char key[8];     /* DES key */
+   int buf_size;
    char *replybuff;
    char *replyfile;
    };
@@ -81,12 +96,14 @@ struct cfd_thread_arg
 /* GLOBAL VARIABLES                                                */
 /*******************************************************************/
 
-int CLOCK_DRIFT = 1000;
+int CLOCK_DRIFT = 3600;   /* 1hr */
 int CFD_MAXPROCESSES = 0;
 int ACTIVE_THREADS = 0;
 int NO_FORK = false;
 int MULTITHREAD = false;
+int CHECK_RFC931 = false;
 int CFD_INTERVAL = 0;
+int DENYBADCLOCKS = true;
 
 struct option CFDOPTIONS[] =
    {
@@ -103,7 +120,6 @@ struct option CFDOPTIONS[] =
 /* PARSER                                                          */
 /*******************************************************************/
 
-char CURRENTPATH[bufsize];
 char CFRUNCOMMAND[bufsize];
 time_t CFDSTARTTIME;
 
