@@ -84,7 +84,7 @@ if (i < MAXCHILD)
    {
    if ( fork() == 0  ) /* child */
       {
-      printf("cfrun: (%s) Thread %d\n",ip->name, i);
+      printf("cfrun: (%s)[%d]\n",ip->name, i);
       Debug("pid = %d i = %d\n", getpid(), i);
       if (ConnectToServer(ip->name,ip->classes, FileFlag))
 	 {
@@ -422,6 +422,13 @@ while (true)
       break;
       }
 
+   if ((sp = strstr(recvbuffer,"BAD:")) != NULL)
+      {
+      *sp = '\0';
+      fprintf(fp,"%s",recvbuffer);
+      break;
+      }   
+
    if (n_read == 0)
       {
       break;
@@ -576,5 +583,10 @@ FatalError()
 exit(1);
 }
 
+AddMultipleClasses()
+{
+ /* hack */
+}
+
 /* EOF */
 
