@@ -29,7 +29,8 @@
 /*                                                                 */
 /*  GLOBAL class default variables for cfengine                    */
 /*  These variables are what needs to be modified if you add or    */
-/*  modify class definitions...                                    */
+/*  modify class definitions... remember also to change clsattr    */
+/*  and search for the os types in cfengine.c (mount stuff)        */
 /*                                                                 */
 /*******************************************************************/
 
@@ -105,7 +106,7 @@ char *CLASSATTRIBUTES[clssattr][attrdim] =
    "nextstep","*","*",         /* nextstep */
    "sn*","cray*","*",          /* cray */
    "gnu*","*","*",             /* gnu */
-   "Windows_NT","*","*",       /* NT */
+   "cygwin_nt*","*","*",       /* NT */
    "unix_sv","*","*",          /* Unixware */
    "openbsd","*","*",          /* OpenBSD */
    "unused1","blah","blah",
@@ -143,7 +144,7 @@ char *VPSCOMM[clssattr] =
    "/bin/ps",       /* nextstep */
    "/bin/ps",       /* cray */
    "/bin/ps",       /* gnu */
-   "/C/usr/bin/ps", /* NT */
+   "/bin/ps",       /* NT - cygnus */
    "/bin/ps",       /* unixware */
    "/bin/ps",       /* openbsd */
    "/bin/ps",
@@ -198,7 +199,7 @@ char *VMOUNTCOMM[clssattr] =
    "",
    "/etc/mount -va",      /* sun4 */
    "/etc/mount -va",      /* ultrix */
-   "/sbin/mount -a",      /* hpux10 */
+   "/sbin/mount -ea",      /* hpux10 */
    "/etc/mount -va",      /* hpux */
    "/etc/mount -t nfs",   /* aix */
    "/bin/mount -va",      /* linux */
@@ -219,9 +220,9 @@ char *VMOUNTCOMM[clssattr] =
    "/usr/etc/mount -a",   /* nextstep */
    "/etc/mount -va",      /* cray */
    "/bin/mount -va",      /* gnu */
-   "N/A",                 /* NT */
+   "/bin/sh /etc/fstab",  /* NT - possible security issue */
    "/sbin/mountall",      /* Unixware */
-   "/sbin/umount",        /* openbsd */
+   "/sbin/mount",         /* openbsd */
    "unused-blah",
    "unused-blah",
    "unused-blah",
@@ -257,7 +258,7 @@ char *VUNMOUNTCOMM[clssattr] =
    "/usr/etc/umount",  /* nextstep */
    "/etc/umount",      /* cray */
    "/sbin/umount",     /* gnu */
-   "N/A",              /* NT */
+   "/bin/umount",      /* NT */
    "/sbin/umount",     /* Unixware */
    "/sbin/umount",     /* openbsd */
    "unused-blah",
@@ -297,7 +298,7 @@ char *VMOUNTOPTS[clssattr] =
    "bg,hard,intr",    /* nextstep */
    "bg,hard,intr",    /* cray */
    "defaults",        /* gnu */
-   "N/A",             /* NT */
+   "",                /* NT */
    "bg,hard,intr",    /* Unixware */
    "-i,-b",           /* openbsd */
    "unused-blah",
@@ -374,7 +375,7 @@ char *VRESOLVCONF[clssattr] =
    "/etc/resolv.conf",     /* nextstep */
    "/etc/resolv.conf",     /* cray */
    "/etc/resolv.conf",     /* gnu */
-   "/C/resolv.conf",       /* NT */
+   "/etc/resolv.conf",     /* NT */
    "/etc/resolv.conf",     /* Unixware */
    "/etc/resolv.conf",     /* openbsd */
    "unused-blah",
@@ -414,7 +415,7 @@ char *VFSTAB[clssattr] =
    "/etc/fstab",       /* nextstep */
    "/etc/fstab",       /* cray */
    "/etc/fstab",       /* gnu */
-   "N/A",              /* NT */
+   "/etc/fstab",       /* NT */
    "/etc/vfstab",      /* Unixware */
    "/etc/fstab",       /* openbsd */
    "unused-blah",
