@@ -38,27 +38,27 @@
 
 /* Protocol specs:
 
-   Stat file:  AUTH client-name server-name
-               SYNCH long-time-string STAT filename
+   Pre authorization: CAUTH (clear text auth) client-name server-name user-name
 
+   Stat file:  SYNCH long-time-string STAT filename
 	       reply with OK: <stat-reply-string>
+	       keep open channel
 
-   Get file:   AUTH client-name server-name
-               GET filename
+   md5 file    Checksum compare
+
+   Get file:   GET filename
 
 	       reply with <multiple buffers>, break on zero length buffer
 	       or BAD: <message>
+	       keep open channel for more stat/get
 
-   Opendir:    AUTH client-name server-name
-               OPENDIR dir-name
+   Opendir:    OPENDIR dir-name
 
 	       reply with <multiple buffers>, break on zero length buffer
 
-   Exec:       AUTH client-name server-name
-               EXEC option-string
-
+   Exec:       EXEC option-string
    	       reply with <multiple buffers>, break on zero length buffer
-
+               Close after this.
 
 Replies:
 
@@ -72,6 +72,90 @@ Replies:
 /* Level 1                                                           */
 /*********************************************************************/
 
+OpenServerConnection(serverlist,ip)
+
+struct Item *serverlist;
+struct Image *ip;
+
+{ int TimeOut(),err;
+  struct sockaddr_in cin;
+
+if (strcmp(ip->server,"localhost") == 0)
+   {
+   AUTHENTICATED = true;
+   return true;
+   }
+
+AUTHENTICATED = false; 
+ 
+if (SD == cf_not_connected)
+   {
+   Debug("Opening server connnection to %s\n",ip->server);
+   bzero(&cin,sizeof(cin));
+
+   cin.sin_port = PORTNUMBER;
+   cin.sin_addr.s_addr = (ip->dns)->s_addr;
+   cin.sin_family = AF_INET; 
+   
+   Debug("Trying to connect to %s = %s, port h=%d\n",ip->server,inet_ntoa(cin.sin_addr),PORTNUMBER);
+  
+   if ((SD = socket(AF_INET,SOCK_STREAM,0)) == -1)
+      {
+      CfLog(cferror,"","socket");
+      AUTHENTICATED = false;
+      return false;
+      }
+   
+   signal(SIGALRM,(void *)TimeOut);
+   alarm(CF_TIMEOUT);
+   
+   if (err=connect(SD,(void *) &cin,sizeof(cin)) == -1)
+      {
+      sprintf(OUTPUT,"Couldn't connect to host %s\n",ip->server);
+      CfLog(cfinform,OUTPUT,"connect");
+      CloseServerConnection();
+      AUTHENTICATED = false;
+      return false;
+      }
+   
+   alarm(0);
+   signal(SIGALRM,SIG_DFL);
+   
+   Debug("Sending on socket...%d\n",SD);
+
+   if (! IdentifyForVerification(SD,VFQNAME))
+      {
+      CfLog(cferror,"Server registration procedure failed","");
+      errno = EPERM;
+      CloseServerConnection();
+      AUTHENTICATED = false;
+      return false;
+      }
+
+   AUTHENTICATED = true;
+   return true;
+   }
+else
+   {
+   Debug("Server connection to %s already open on %d\n",ip->server,SD);
+   }
+
+AUTHENTICATED = true; 
+return true; 
+}
+
+/*********************************************************************/
+
+CloseServerConnection()
+
+{
+Debug("Closing current connection\n");
+close(SD);
+SD = cf_not_connected;
+}
+
+/*********************************************************************/
+
 cf_rstat(file,buf,ip,stattype)
 
 /* If a link, this reads readlink and sends it back in the same
@@ -82,16 +166,22 @@ struct stat *buf;
 struct Image *ip;
 char *stattype;
 
-{ struct sockaddr_in cin;
-  char sendbuffer[bufsize];
+{ char sendbuffer[bufsize];
   char recvbuffer[bufsize];
+  char in[bufsize],out[bufsize];
   struct cfstat cfst;
-  int sd,i,err,TimeOut(),ret;
+  int i,ret;
   time_t tloc;
   struct hostent *hp;
 
 Debug("cf_rstat(%s)\n",file);
 
+if (strlen(file) > bufsize-30)
+   {
+   CfLog(cferror,"Filename too long","");
+   return -1;
+   }
+ 
 ret = GetCachedStatData(file,buf,ip,stattype);
 
 if (ret != 0)
@@ -99,82 +189,57 @@ if (ret != 0)
    return ret;
    }
 
-bzero(&cin,sizeof(cin));
-
-cin.sin_port = PORTNUMBER;
-cin.sin_addr.s_addr = (ip->dns)->s_addr;
-cin.sin_family = AF_INET; 
-
-Debug("Trying to connect to %s = %s, port h=%d\n",ip->server,inet_ntoa(cin.sin_addr),PORTNUMBER);
-
-if ((sd = socket(AF_INET,SOCK_STREAM,0)) == -1)
-   {
-   CfLog(cferror,"","socket");
-   exit(1);
-   }
-
-signal(SIGALRM,(void *)TimeOut);
-alarm(CF_TIMEOUT);
-
-if (err=connect(sd,(void *) &cin,sizeof(cin)) == -1)
-   {
-   sprintf(OUTPUT,"Couldn't connect to host %s\n",ip->server);
-   CfLog(cfinform,OUTPUT,"connect");
-   close(sd);
-   return -1;
-   }
-
-alarm(0);
-signal(SIGALRM,SIG_DFL);
-
-Debug("Sending on socket...\n");
-
-if (! IdentifyForVerification(sd,VFQNAME))
-   {
-   CfLog(cferror,"Server registration procedure failed","");
-   errno = EPERM;
-   close(sd);
-   return -1;
-   }
-
 if ((tloc = time((time_t *)NULL)) == -1)
    {
    CfLog(cferror,"Couldn't read system clock\n","");
    }
 
-sprintf(sendbuffer,"SYNCH %d STAT %s",tloc,file);
-
-if (send(sd,sendbuffer,bufsize,0) == -1)
+if (ip->secure)
    {
-   CfLog(cferror,"","send");
-   close(sd);
+#ifdef HAVE_LIBCRYPTO
+   bzero(in,bufsize);
+   bzero(out,bufsize);
+   bzero(sendbuffer,bufsize);
+   sprintf(in,"SYNCH %d STAT %s",tloc,file);
+   cfencrypt(in,out,CFDES1,CFDES2,CFDES3,strlen(in));
+   sprintf(sendbuffer,"SSYNCH %d",strlen(in));
+   bcopy(out,sendbuffer+23,bufsize-24);
+#endif   
+   }
+else
+   {
+   sprintf(sendbuffer,"SYNCH %d STAT %s",tloc,file);
+   }
+
+if (send(SD,sendbuffer,bufsize,0) == -1)
+   {
+   CfLog(cferror,"Can't send on socket","send");
    return -1;
    }
 
-if (RecvSocketStream(sd,recvbuffer,bufsize,0) == -1)
+if (RecvSocketStream(SD,recvbuffer,bufsize,0) == -1)
    {
-   close(sd);
    return -1;
    }
 
 if (strstr(recvbuffer,"unsynchronized"))
    {
    CfLog(cferror,"Clocks differ too much to do copy by date (security)","");
+   CfLog(cferror,recvbuffer+4,"");
+   return -1;
    }
 
 if (strncmp(recvbuffer,"BAD",3) == 0)
    {
    sprintf(OUTPUT,"Server returned error: %s\n",recvbuffer+4);
    CfLog(cfverbose,OUTPUT,"");
-   close(sd);
    errno = EPERM;
    return -1;
    }
 
-
 if (strncmp(recvbuffer,"OK:",3) == 0)
    {
-   int d1,d2,d3,d4,d5,d6,d7,d8,d9,d10,d11,d12;
+   int d1,d2,d3,d4,d5,d6,d7,d8,d9,d10,d11,d12=0;
    
    sscanf(recvbuffer,"OK: %1d %5d %12d %12d %12d %12d %12d %12d %12d %12d %12d %12d",
 	  &d1,&d2,&d3,&d4,&d5,&d6,&d7,&d8,&d9,&d10,&d11,&d12);
@@ -194,7 +259,6 @@ if (strncmp(recvbuffer,"OK:",3) == 0)
 
    /* Use %?d here to avoid memory overflow attacks */
 
-   Debug("RECV:%s\n",recvbuffer);
    Debug("Mode = %d,%d\n",d2,d3);
    
    Debug("OK: type=%d\n mode=%o\n lmode=%o\n uid=%d\n gid=%d\n size=%ld\n atime=%d\n mtime=%d ino=%d nlnk=%d\n",
@@ -203,9 +267,8 @@ if (strncmp(recvbuffer,"OK:",3) == 0)
 
    bzero(recvbuffer,bufsize);
    
-   if (RecvSocketStream(sd,recvbuffer,bufsize,0) == -1)
+   if (RecvSocketStream(SD,recvbuffer,bufsize,0) == -1)
       {
-      close(sd);
       return -1;
       }
    
@@ -272,15 +335,14 @@ if (strncmp(recvbuffer,"OK:",3) == 0)
    buf->st_ino   = cfst.cf_ino;
    buf->st_nlink = cfst.cf_nlink;
    
-   close(sd);
    return 0;
    }
 
 
-CfLog(cfinform,"Server didn't respond correctly. Got:",recvbuffer);
+sprintf(OUTPUT,"Protocol error in stat (shouldn't happen). Got [%s]\n",recvbuffer); 
+CfLog(cfinform,OUTPUT,"");
 errno = EPERM;
 
-close(sd);
 return -1;
 }
 
@@ -291,46 +353,17 @@ CFDIR *cf_ropendir(dirname,ip)
 char *dirname;
 struct Image *ip;
 
-{ struct sockaddr_in cin;
-  char sendbuffer[bufsize];
+{ char sendbuffer[bufsize];
   char recvbuffer[bufsize];
-  int sd,i,err,TimeOut(),offset,n;
+  int i,offset,n, done=false;
   CFDIR *cfdirh;
   char *sp;
 
 Debug("CfOpenDir(%s:%s)\n",ip->server,dirname);
 
-bzero(&cin,sizeof(cin));
-
-cin.sin_port = PORTNUMBER;
-cin.sin_addr.s_addr = (ip->dns)->s_addr;
-cin.sin_family = AF_INET; 
-
-if ((sd = socket(AF_INET,SOCK_STREAM,0)) == -1)
+if (strlen(dirname) > bufsize - 20)
    {
-   CfLog(cferror,"","socket");
-   exit(1);
-   }
-
-signal(SIGALRM,(void *)TimeOut);
-alarm(CF_TIMEOUT);
-
-if (err=connect(sd,(void *) &cin,sizeof(cin)) == -1)
-   {
-   sprintf(OUTPUT,"Couldn't connect to host %s\n",ip->server);
-   CfLog(cfinform,OUTPUT,"connect");
-   close(sd);
-   return NULL;
-   }
-
-alarm(0);
-signal(SIGALRM,SIG_DFL);
-
-if (! IdentifyForVerification(sd,VFQNAME))
-   {
-   CfLog(cfinform,"Server registration procedure failed\n","");
-   errno = EPERM;
-   close(sd);
+   CfLog(cferror,"Directory name too long");
    return NULL;
    }
 
@@ -346,22 +379,19 @@ cfdirh->cf_dirh = NULL;
 
 sprintf(sendbuffer,"OPENDIR %s",dirname);
 
-if (send(sd,sendbuffer,bufsize,0) == -1)
+if (send(SD,sendbuffer,bufsize,0) == -1)
    {
-   CfLog(cferror,"","send");
-   close(sd);
    return NULL;
    }
 
-while (true)
+while (!done)
    {
-   if ((n = RecvSocketStream(sd, recvbuffer, bufsize,0)) == -1)
+   if ((n = RecvSocketStream(SD,recvbuffer,bufsize,0)) == -1)
       {
       if (errno == EINTR) 
          {
          continue;
          }
-      close(sd);
       return false;
       }
 
@@ -370,16 +400,30 @@ while (true)
       break;
       }
 
-   if (strncmp(recvbuffer,"BAD:",4) == 0) /* opendir failed */
+   if (strncmp(recvbuffer,CFFAILEDSTR,strlen(CFFAILEDSTR)) == 0)
       {
-      Verbose("cfengine: %s",recvbuffer+4);
-      continue;
+      sprintf(OUTPUT,"Network access to %s:%s denied\n",ip->server,dirname);
+      CfLog(cfinform,OUTPUT,"");
+      return false;      
+      }
+
+   if (strncmp(recvbuffer,"BAD:",4) == 0)
+      {
+      sprintf(OUTPUT,"%s\n",recvbuffer+4);
+      CfLog(cfinform,OUTPUT,"");
+      return false;      
       }
 
    for (sp = recvbuffer; *sp != '\0'; sp++)
       {
-      AppendItem(&(cfdirh->cf_list),sp,NULL);
+      if (strncmp(sp,CFD_TERMINATOR,strlen(CFD_TERMINATOR)) == 0)        /* End transmission */
+	 {
+	 cfdirh->cf_listpos = cfdirh->cf_list;
+	 return cfdirh;
+	 }
 
+      AppendItem(&(cfdirh->cf_list),sp,NULL);
+      
       while(*sp != '\0')
 	 {
 	 sp++;
@@ -387,8 +431,6 @@ while (true)
       }
    }
 
-close(sd);
-
 cfdirh->cf_listpos = cfdirh->cf_list;
 return cfdirh;
 }
@@ -415,45 +457,10 @@ char *file1, *file2;
 struct Image *ip;
 
 { static unsigned char d[16];
-  struct sockaddr_in cin;
   char sendbuffer[bufsize];
-  char recvbuffer[bufsize];
-  int sd,i,err,TimeOut(),offset;
+  char recvbuffer[bufsize],in[bufsize],out[bufsize];
+  int i,offset;
   
-bzero(&cin,sizeof(cin));
-
-cin.sin_port = PORTNUMBER;
-cin.sin_addr.s_addr = (ip->dns)->s_addr;
-cin.sin_family = AF_INET; 
-
-if ((sd = socket(AF_INET,SOCK_STREAM,0)) == -1)
-   {
-   CfLog(cferror,"","socket");
-   Verbose("No answer from host, assuming checksum ok to avoid remote copy for now...\n");
-   return false;
-   }
-
-signal(SIGALRM,(void *)TimeOut);
-alarm(CF_TIMEOUT);
-
-if (err=connect(sd,(void *) &cin,sizeof(cin)) == -1)
-   {
-   CfLog(cfinform,"No answer from host, assuming checksum ok to avoid remote copy for now...\n","");
-   close(sd);
-   return false;
-   }
-
-alarm(0);
-signal(SIGALRM,SIG_DFL);
-
-if (! IdentifyForVerification(sd,VFQNAME))
-   {
-   CfLog(cfinform,"Server registration procedure failed\n","");
-   errno = EPERM;
-   close(sd);
-   return true;
-   }
-
 cfMDFile(file2,d);   /* send this to the server for comparison */
 
 bzero(recvbuffer,bufsize);
@@ -462,23 +469,37 @@ bzero(sendbuffer,bufsize);
 if (20 + 3*16 + strlen(file1) > bufsize-1)
    {
    CfLog(cferror,"MD5 transfer overflow...updating for safety\n","");
-   close(sd);
    return true;
    }
 
-sprintf(sendbuffer,"MD5 %s %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d \0",file1,
-	d[0],d[1],d[2],d[3],d[4],d[5],d[6],d[7],d[8],d[9],d[10],d[11],d[12],d[13],d[14],d[15]);
 
-if (send(sd,sendbuffer,bufsize,0) == -1)
+if (ip->secure)
+   {
+#ifdef HAVE_LIBCRYPTO
+   bzero(in,bufsize);
+   bzero(out,bufsize);
+   bzero(sendbuffer,bufsize);
+   sprintf(in,"MD5 %s\n %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d \0",file1,
+	   d[0],d[1],d[2],d[3],d[4],d[5],d[6],d[7],d[8],d[9],d[10],d[11],d[12],d[13],d[14],d[15]);
+   cfencrypt(in,out,CFDES1,CFDES2,CFDES3,strlen(in));
+   sprintf(sendbuffer,"SMD5 %d",strlen(in));
+   bcopy(out,sendbuffer+23,bufsize-24);
+#endif   
+   }
+else
+   {
+   sprintf(sendbuffer,"MD5 %s\n %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d \0",file1,
+	   d[0],d[1],d[2],d[3],d[4],d[5],d[6],d[7],d[8],d[9],d[10],d[11],d[12],d[13],d[14],d[15]);
+   } 
+ 
+if (send(SD,sendbuffer,bufsize,0) == -1)
    {
    CfLog(cferror,"","send");
-   close(sd);
    return false;
    }
 
-if (RecvSocketStream(sd, recvbuffer, bufsize,0) == -1)
+if (RecvSocketStream(SD,recvbuffer,bufsize,0) == -1)
    {
-   close(sd);
    Verbose("No answer from host, assuming checksum ok to avoid remote copy for now...\n");
    return false;
    }
@@ -486,13 +507,11 @@ if (RecvSocketStream(sd, recvbuffer, bufsize,0) == -1)
 if (strcmp(CFD_TRUE,recvbuffer) == 0)
    {
    Debug("MD5 mismatch: %s\n",recvbuffer);
-   close(sd);
    return true; /* mismatch */
    }
 else
    {
    Debug("MD5 match: %s\n",recvbuffer);
-   close(sd);
    return false;
    }
  
@@ -501,69 +520,57 @@ else
 
 /*********************************************************************/
 
-CopyRegNet(source,new,ip)
+CopyRegNet(source,new,ip,size)
 
 char *source, *new;
 struct Image *ip;
+off_t size;
 
-{ int sd, dd, buf_size, TimeOut();
-  char *buf, *cp, sendbuffer[bufsize];
-  int n_read, *intp, err;
+{ int dd, buf_size, TimeOut(), i=0;
+  char *buf,*cp,*sp,in[bufsize],out[bufsize],sendbuffer[bufsize],key[8];
+  int n_read, *intp,err,toget,towrite,len;
   long n_read_total = 0;
-  int last_write_made_hole = 0;
-  struct sockaddr_in cin;
+  int last_write_made_hole = 0, done = false;
 
-Debug("CopyRegNet(%s,%s)\n",source,new);
-  
-bzero(&cin,sizeof(cin));
+Debug("CopyRegNet(%s,%s,%d) to socket %d\n",source,new,size,SD);
 
-cin.sin_port = PORTNUMBER;
-cin.sin_addr.s_addr = (ip->dns)->s_addr;
-cin.sin_family = AF_INET; 
-
-if ((dd = open(new,O_WRONLY | O_CREAT | O_TRUNC, 0600)) == -1)
+if ((strlen(new) > bufsize-20))
    {
-   sprintf(OUTPUT,"Can't copy to destination %s!\n",new);
+   CfLog(cferror,"Filename too long","");
+   return false;
+   }
+ 
+unlink(new);  /* To avoid link attacks */ 
+  
+if ((dd = open(new,O_WRONLY|O_CREAT|O_TRUNC|O_EXCL, 0600)) == -1)
+   {
+   sprintf(OUTPUT,"Copy %s:%s security - failed attempt to exploit a race? (Not copied)\n",ip->server,new);
    CfLog(cferror,OUTPUT,"open");
    unlink(new);
    return false;
    }
 
-if ((sd = socket(AF_INET,SOCK_STREAM,0)) == -1)
+if (ip->secure)
    {
-   CfLog(cferror,"","socket");
-   exit(1);
+#ifdef HAVE_LIBCRYPTO
+   des_random_key(key);
+   bzero(in,bufsize);
+   bzero(out,bufsize);
+   bzero(sendbuffer,bufsize);
+   sprintf(in,"GET %8.8s %s",key,source);
+   cfencrypt(in,out,CFDES1,CFDES2,CFDES3,strlen(in));
+   sprintf(sendbuffer,"SGET %4d %4d",strlen(in),ST_BLKSIZE(dstat));
+   bcopy(out,sendbuffer+31,bufsize-32);
+#endif
+   }
+else
+   {
+   sprintf(sendbuffer,"GET %d %s",ST_BLKSIZE(dstat),source);
    }
 
-signal(SIGALRM,(void *)TimeOut);
-alarm(CF_TIMEOUT);
-
-if (err = connect(sd,(void *) &cin,sizeof(cin)) == -1)
+if (send(SD,sendbuffer,bufsize,0) == -1)
    {
-   sprintf(OUTPUT,"Couldn't connect to host %s\n",ip->server);
-   CfLog(cferror,OUTPUT,"connect");
-   close(sd);
-   close(dd);
-   return false;
-   }
-
-alarm(0);
-signal(SIGALRM,SIG_DFL);
-
-if (! IdentifyForVerification(sd,VFQNAME))
-   {
-   CfLog("Server registration procedure failed\n","");
-   close(sd);
-   close(dd);
-   return false;
-   }
-
-sprintf(sendbuffer,"GET %s",source);
-
-if (send(sd,sendbuffer,bufsize,0) == -1)
-   {
-   CfLog(cferror,"","send");
-   close(sd);
+   CfLog(cferror,"Couldn't send","send");
    close(dd);
    return false;
    }
@@ -571,16 +578,31 @@ if (send(sd,sendbuffer,bufsize,0) == -1)
 buf_size = ST_BLKSIZE(dstat);
 buf = (char *) malloc(buf_size + sizeof(int));
 
-while (true)
+n_read_total = 0;
+
+while (!done)
    {
-   if ((n_read = recv(sd, buf, buf_size,0)) == -1)
+   if ((size - n_read_total)/buf_size > 0)
+      {
+      toget = towrite = buf_size;
+      }
+   else if (size != 0)
+      {
+      towrite = (size - n_read_total);
+      toget = buf_size; /* padding required for encryption chaining */
+      }
+   else
+      {
+      toget = towrite = 0;
+      }
+
+   if ((n_read = RecvSocketStream(SD,buf,toget,0)) == -1)
       {
       if (errno == EINTR) 
          {
          continue;
          }
 
-      close(sd);
       close(dd);
       free(buf);
       return false;
@@ -589,20 +611,39 @@ while (true)
    if (strncmp(buf,CFFAILEDSTR,strlen(CFFAILEDSTR)) == 0)
       {
       sprintf(OUTPUT,"Network access to %s:%s denied\n",ip->server,source);
+      RecvSocketStream(SD,buf,bufsize-toget,0); /* flush rest of transaction */
       CfLog(cfinform,OUTPUT,"");
-      close(sd);
       close(dd);
       free(buf);
-      AUTHENTICATED = false;
       return false;      
       }
 
+   if (strncmp(buf,"BAD:",4) == 0) /* error */
+      {
+      CfLog(cfinform,buf+4,"");
+      close(dd);
+      free(buf);
+      return false;      
+      }
+
+   if (ip->secure)
+      {
+      bzero(sendbuffer,bufsize);
+      cfdecrypt(buf,sendbuffer,CFDES1,key,CFDES3,buf_size);
+      bcopy(sendbuffer,buf,buf_size);
+      }
+
    if (n_read == 0)
       {
       break;
       }
 
-   n_read_total += n_read;
+   n_read_total += towrite; /*n_read;*/
+
+   if (n_read_total >= (long)size)  /* Handle EOF without closing socket */
+      {                                 /* signature is 8 bytes */
+      done = true;
+      }
 
    intp = 0;
 
@@ -632,13 +673,12 @@ while (true)
       if (cp > buf + n_read)
          {
          /* Make a hole.  */
-         if (lseek (dd, (off_t) n_read, SEEK_CUR) < 0L)
+         if (lseek (dd,(off_t)n_read,SEEK_CUR) < 0L)
             {
             sprintf (OUTPUT,"lseek in CopyReg, dest=%s\n", new);
             CfLog(cferror,OUTPUT,"lseek");
             free(buf);
 	    unlink(new);
-            close(sd);
 	    close(dd);
             return false;
             }
@@ -653,11 +693,10 @@ while (true)
 
    if (intp == 0)
       {
-      if (cf_full_write (dd, buf, n_read) < 0)
+      if (cf_full_write (dd,buf,towrite) < 0)
          {
-	 sprintf(OUTPUT,"Full_write failed in CopyReg(%s:%s,%s)\n",ip->server,source,new);
+	 sprintf(OUTPUT,"CopyReg(%s:%s,%s) failed\n",ip->server,source,new);
          CfLog(cferror,OUTPUT,"");
-         close(sd);
          close(dd);
          free(buf);
 	 unlink(new);
@@ -675,20 +714,17 @@ while (true)
     {
     /* Write a null character and truncate it again.  */
 
-    if (cf_full_write (dd, "", 1) < 0 || ftruncate (dd, n_read_total) < 0)
+    if (cf_full_write (dd,"",1) < 0 || ftruncate (dd,n_read_total) < 0)
        {
        CfLog(cferror,"cfengine: full_write or ftruncate error in CopyReg\n","");
        free(buf);
        unlink(new);
-       close(sd);
        close(dd);
        return false;
        }
     }
 
-close(sd);
 close(dd);
-
 free(buf);
 return true;
 }
@@ -715,6 +751,7 @@ for (sp = ip->cache; sp != NULL; sp=sp->next)
       if (sp->cf_failed)  /* cached failure from cfopendir */
 	 {
 	 errno = EPERM;
+	 Debug("Cached failure to stat\n");
 	 return -1;
 	 }
 
@@ -736,10 +773,12 @@ for (sp = ip->cache; sp != NULL; sp=sp->next)
       statbuf->st_ino   = sp->cf_ino;
       statbuf->st_nlink = sp->cf_nlink;      
 
+      Debug("Found in cache\n");
       return true;
       }
    }
 
+Debug("Did not find in cache\n"); 
 return false;
 }
 
