@@ -57,13 +57,13 @@ Debug("FixCompressedArrayValue(%d,%s)\n",i,value);
 
 if ((ap = (struct CompressedArray *)malloc(sizeof(struct CompressedArray))) == NULL)
    {
-   perror("Can't allocate memory in SetCompressedArray()");
+   CfLog(cferror,"Can't allocate memory in SetCompressedArray()","malloc");
    FatalError("");
    }
 
 if ((sp = malloc(strlen(value)+2)) == NULL)
    {
-   perror("Can't allocate memory in SetCompressedArray()");
+   CfLog(cferror,"Can't allocate memory in SetCompressedArray()","malloc");
    FatalError("");
    }
 
