@@ -73,7 +73,7 @@ for (ip = VMOUNTLIST; ip != NULL; ip=ip->next)
 
    for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
       {
-      if (!SensibleFile(dirp->d_name,ip->name))
+      if (!SensibleFile(dirp->d_name,ip->name,NULL))
          {
          continue;
          }
@@ -96,7 +96,7 @@ for (ip = VMOUNTLIST; ip != NULL; ip=ip->next)
 
       for (dirp2 = readdir(dirh2); dirp2 != NULL; dirp2 = readdir(dirh2))
          {
-         if (!SensibleFile(dirp2->d_name,homedir))
+         if (!SensibleFile(dirp2->d_name,homedir,NULL))
 	    {
 	    continue;
 	    }
@@ -127,7 +127,7 @@ for (ip = VMOUNTLIST; ip != NULL; ip=ip->next)
 
          uid = statbuf.st_uid;
 
-         DoEditFile(ptr,dest);
+         WrapDoEditFile(ptr,dest);
       
          chown(dest,uid,sameowner);
          }
@@ -139,6 +139,73 @@ for (ip = VMOUNTLIST; ip != NULL; ip=ip->next)
 
 /********************************************************************/
 
+WrapDoEditFile(ptr)
+
+struct Edit *ptr;
+
+{ struct stat statbuf,statbuf2;
+  char linkname[bufsize];
+  char realname[bufsize];
+
+Debug("WrapDoEditFile(%s)\n",ptr->fname);
+  
+if (lstat(ptr->fname,&statbuf) != -1)
+   {
+   if (S_ISLNK(statbuf.st_mode))
+      {
+      Verbose("%s: File %s is a link, editing real file instead\n",VPREFIX,ptr->fname);
+      
+      bzero(linkname,bufsize);
+      bzero(realname,bufsize);
+      
+      if (readlink(ptr->fname,linkname,bufsize) == -1)
+	 {
+	 sprintf(OUTPUT,"Cannot read link %s\n",ptr->fname);
+	 CfLog(cferror,OUTPUT,"readlink");
+	 return;
+	 }
+      
+      if (linkname[0] != '/')
+	 {
+	 strcpy(realname,ptr->fname);
+	 ChopLastNode(realname);
+	 AddSlash(realname);
+	 }
+      
+      if (BufferOverflow(realname,linkname))
+	 {
+	 sprintf(OUTPUT,"(culprit %s in editfiles)\n",ptr->fname);
+	 CfLog(cferror,OUTPUT,"");
+	 return;
+	 }
+      
+      if (stat(ptr->fname,&statbuf2) != -1)
+	 {
+	 if (statbuf2.st_uid != statbuf.st_uid)
+	    {
+	    /* Link to /etc/passwd? ouch! */
+	    sprintf(OUTPUT,"Forbidden to edit a link to another user's file with privilege (%s)",ptr->fname);
+	    CfLog(cfinform,OUTPUT,"");
+	    return;
+	    }
+	 }
+      
+      strcat(realname,linkname);
+      DoEditFile(ptr,realname);
+      }
+   else
+      {
+      DoEditFile(ptr,ptr->fname);
+      }
+   }
+else
+   {
+   DoEditFile(ptr,ptr->fname);
+   }
+}
+
+/********************************************************************/
+
 DoEditFile(ptr,filename)
 
 struct Edit *ptr;
@@ -156,7 +223,8 @@ char *filename;
   int todo = 0;
   int foreachloop = false;
   FILE *loop_fp;
-  
+
+Debug("DoEditFile(%s)\n",filename);
 filestart = NULL;
 currenteditscript = NULL;
 searchstr = NULL;
@@ -186,7 +254,7 @@ CheckEditSwitches(filename,ptr->actions);
 
 if (! LoadItemList(&filestart,filename))
    {
-   CfLog(cfinform,"File was marked for editing\n","");
+   CfLog(cfverbose,"File was marked for editing\n","");
    ReleaseCurrentLock();
    return;
    }
@@ -808,7 +876,7 @@ while (ep != NULL)
 
       case DefineClasses:
 	       break;
-	       
+
       default: printf("%s: Unknown action in editing of file %s\n",VPREFIX,filename);
                break;
       }
@@ -823,6 +891,11 @@ EditVerbose("...................................................................
 
 EDITVERBOSE = false;
 
+if (DONTDO || CompareToFile(filestart,filename))
+   {
+   NUMBEROFEDITS = 0;
+   }
+ 
 if ((! DONTDO) && (NUMBEROFEDITS > 0))
    {
    SaveItemList(filestart,filename);
