@@ -234,8 +234,6 @@ if (IsItemIn(VHEAP,class))
    return;
    }
 
-Debug(">>Pushing class %s on heap\n",class);
-
 AppendItem(&VHEAP,class,NULL);
 }
 
@@ -246,8 +244,6 @@ DeleteClassFromHeap(class)
 char *class;
 
 {
-Debug(">>Deleting class %s from heap\n",class);
-
 DeleteItemMatching(&VHEAP,class);
 }
 
@@ -259,13 +255,10 @@ char *sp;
 
 { int i;
 
-Debug("IsHardClass(%s)",sp);
-
 for (i = 2; CLASSTEXT[i] != '\0'; i++)
    {
    if (strcmp(CLASSTEXT[i],sp) == 0)
       {
-      Debug("CLASS = %s true\n",CLASSTEXT[i]);
       return(true);
       }
    }
@@ -274,12 +267,10 @@ for (i = 0; i < 7; i++)
    {
    if (strcmp(DAYTEXT[i],sp)==0)
       {
-      Debug("-false\n");
       return(false);
       }
    }
 
-Debug("-false\n");
 return(false);
 }
 
@@ -455,11 +446,9 @@ EvaluateORString(class,list)
 char *class;
 struct Item *list;
 
-{ char *sp, cbuff[maxvarsize];
+{ char *sp, cbuff[bufsize];
   int result = false;
 
-bzero(cbuff,maxvarsize);
-
 for (sp = class; *sp != '\0'; sp++)
    {
    while (*sp == '|')
@@ -467,7 +456,7 @@ for (sp = class; *sp != '\0'; sp++)
       sp++;
       }
 
-   bzero(cbuff,maxvarsize);
+   bzero(cbuff,bufsize);
 
    sp += GetORAtom(sp,cbuff);
 
@@ -506,12 +495,11 @@ char *class;
 struct Item *list;
 
 { char *sp, *atom;
-  char cbuff[maxvarsize];
+  char cbuff[bufsize];
   int count = 1, brackets = 0;
   int negation = false;
 
 count = CountEvalAtoms(class);
-
 sp = class;
 
 while(*sp != '\0')
@@ -524,7 +512,7 @@ while(*sp != '\0')
       sp++;
       }
 
-   bzero(cbuff,maxvarsize);
+   bzero(cbuff,bufsize);
 
    sp += GetANDAtom(sp,cbuff) + 1;
 
@@ -537,8 +525,8 @@ while(*sp != '\0')
       atom = cbuff+1;
 
       cbuff[strlen(cbuff)-1] = '\0';
-      
-      if (EvaluateORString(cbuff+1,list))
+
+      if (EvaluateORString(atom,list))
 	 {
 	 if (negation)
 	    {
@@ -553,7 +541,7 @@ while(*sp != '\0')
 	 {
 	 if (negation)
 	    {
-	    return true;
+	    count--;
 	    }
 	 else
 	    {
@@ -563,6 +551,10 @@ while(*sp != '\0')
 
       continue;
       }
+   else
+      {
+      atom = cbuff;
+      }
    
    /* End of parenthesis check */
    
