@@ -100,7 +100,7 @@ if ((dirh = opendir(path)) == NULL)
 
 for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
    {
-   if (!SensibleFile(dirp->d_name,path))
+   if (!SensibleFile(dirp->d_name,path,NULL))
       {
       continue;
       }
@@ -149,7 +149,7 @@ if ((dirh = opendir(name)) == NULL)
 
 for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
    {
-   if (!SensibleFile(dirp->d_name,name))
+   if (!SensibleFile(dirp->d_name,name,NULL))
       {
       continue;
       }
@@ -236,19 +236,34 @@ enum fileactions action;
 struct stat *dstat;
 struct File *ptr;
 struct Item *acl_aliases;
-
+ 
 { mode_t newperm = dstat->st_mode;
-  int amroot = true;
+  int amroot = true, fixmode = true;
   char digest[17];
+#if defined HAVE_CHFLAGS
+  u_long newflags;
+#endif
+ 
+Debug("%s: Checking fs-object %s\n",VPREFIX,file);
 
-Verbose("%s: Checking fs-object %s\n",VPREFIX,file);
-Debug("CheckExistingFile(+%o,-%o)\n",plus,minus); 
-
+#if defined HAVE_CHFLAGS
+if (ptr != NULL)
+   {
+   Debug("CheckExistingFile(+%o,-%o,+%o,-%o)\n",plus,minus,ptr->plus_flags,ptr->minus_flags);
+   }
+else
+   {
+   Debug("CheckExistingFile(+%o,-%o)\n",plus,minus);
+   }
+#else
+Debug("CheckExistingFile(+%o,-%o)\n",plus,minus);
+#endif
+ 
 if (ptr != NULL)
    {
    if (IgnoredOrExcluded(files,file,ptr->inclusions,ptr->exclusions))
       {
-      Verbose("Skipping excluded file %s\n",file);
+      Debug("Skipping excluded file %s\n",file);
       return;
       }
    }
@@ -393,7 +408,7 @@ if (CheckOwner(file,action,uidlist,gidlist,dstat))
 
 if ((ptr != NULL) && S_ISREG(dstat->st_mode) && (ptr->checksum == 'm'))
     {
-    Verbose("Checking MD5 integrity of %s\n",file);
+    Debug("Checking MD5 integrity of %s\n",file);
     cfMDFile(file,digest);
     digest[16] = '\0';
     
@@ -427,132 +442,283 @@ if (CheckACLs(file,action,acl_aliases))
       AddMultipleClasses(ptr->defines);
       }
    }
- 
+
+#ifndef HAVE_CHFLAGS
 if (((newperm & 07777) == (dstat->st_mode & 07777)) && (action != touch))    /* file okay */
    {
    Debug("File okay, newperm = %o, stat = %o\n",(newperm & 07777),(dstat->st_mode & 07777));
    return;
    }
-
-Debug("Trying to fix...\n"); 
-
-switch (action)
+#else
+if ((newperm & 07777) == (dstat->st_mode & 07777))    /* file okay */
    {
-   case linkchildren:
+   Debug("File okay, newperm = %o, stat = %o\n",(newperm & 07777),(dstat->st_mode & 07777));
+   fixmode = false;
+   }
+#endif 
 
-   case warnplain:
-                if (S_ISREG(dstat->st_mode))
-                   {
-                   sprintf(OUTPUT,"%s has permission %o\n",file,dstat->st_mode & 07777);
-		   CfLog(cferror,OUTPUT,"");
-                   sprintf(OUTPUT,"[should be %o]\n",newperm & 07777);
-		   CfLog(cferror,OUTPUT,"");
-                   }
-                break;
-   case warndirs:
-                if (S_ISDIR(dstat->st_mode))
-                   {
-                   sprintf(OUTPUT,"%s has permission %o\n",file,dstat->st_mode & 07777);
-		   CfLog(cferror,OUTPUT,"");
-                   sprintf(OUTPUT,"[should be %o]\n",newperm & 07777);
-		   CfLog(cferror,OUTPUT,"");
-                   }
-                break;
-   case warnall:   
-                sprintf(OUTPUT,"%s has permission %o\n",file,dstat->st_mode & 07777);
-		CfLog(cferror,OUTPUT,"");
-                sprintf(OUTPUT,"[should be %o]\n",newperm & 07777);
-		CfLog(cferror,OUTPUT,"");
-                break;
-
-   case fixplain:
-
-                if (S_ISREG(dstat->st_mode))
-                   {
-                   if (! DONTDO)
-                      {
-                      if (chmod (file,newperm & 07777) == -1)
-			 {
-                         sprintf(OUTPUT,"chmod failed on %s\n",file);
-                         CfLog(cferror,OUTPUT,"chmod");
-                         break;
-                         }
-                      }
-
-                   sprintf(OUTPUT,"%s had permission %o, changed it to %o\n",
-			   file,dstat->st_mode & 07777,newperm & 07777);
-		   CfLog(cfinform,OUTPUT,"");
-		   
-                   if (ptr != NULL)
-		      {
-		      AddMultipleClasses(ptr->defines);
-		      }
-                   }
-                break;
-
-   case fixdirs:
-                if (S_ISDIR(dstat->st_mode))
-                   {
-                   if (! DONTDO)
-                      {
-                      if (chmod (file,newperm & 07777) == -1)
-                         {
-                         sprintf(OUTPUT,"chmod failed on %s\n",file);
-			 CfLog(cferror,OUTPUT,"chmod");
-                         break;
-                         }
-                      }
-
-                   sprintf(OUTPUT,"%s had permission %o, changed it to %o\n",
-			   file,dstat->st_mode & 07777,newperm & 07777);
-		   CfLog(cfinform,OUTPUT,"");
-		   
-                   if (ptr != NULL)
-		      {
-		      AddMultipleClasses(ptr->defines);
-		      }
-                   }
-                break;
-
-   case fixall: if (! DONTDO)
-                   {
-                   if (chmod (file,newperm & 07777) == -1)
-		      {
-                      sprintf(OUTPUT,"chmod failed on %s\n",file);
-                      CfLog(cferror,OUTPUT,"chmod");
-                      break;
-                      }
-                   }
-
-                   sprintf(OUTPUT,"%s had permission %o, changed it to %o\n",
-			   file,dstat->st_mode & 07777,newperm & 07777);
-		   CfLog(cfinform,OUTPUT,"");
-
-                   if (ptr != NULL)
-		      {
-                      AddMultipleClasses(ptr->defines);
-		      }
-                break;
-
-   case touch:  if (! DONTDO)
-                   {
-                   if (chmod (file,newperm & 07777) == -1)
-		      {
-                      sprintf(OUTPUT,"chmod failed on %s\n",file);
-                      CfLog(cferror,OUTPUT,"chmod");
-                      break;
-		      }
-                   utime (file,NULL);
-                   }
-                if (ptr != NULL)
+if (fixmode)
+   {
+   Debug("Trying to fix mode...\n"); 
+   
+   switch (action)
+      {
+      case linkchildren:
+	  
+      case warnplain:
+	  if (S_ISREG(dstat->st_mode))
+	     {
+	     sprintf(OUTPUT,"%s has permission %o\n",file,dstat->st_mode & 07777);
+	     CfLog(cferror,OUTPUT,"");
+	     sprintf(OUTPUT,"[should be %o]\n",newperm & 07777);
+	     CfLog(cferror,OUTPUT,"");
+	     }
+	  break;
+      case warndirs:
+	  if (S_ISDIR(dstat->st_mode))
+	     {
+	     sprintf(OUTPUT,"%s has permission %o\n",file,dstat->st_mode & 07777);
+	     CfLog(cferror,OUTPUT,"");
+	     sprintf(OUTPUT,"[should be %o]\n",newperm & 07777);
+	     CfLog(cferror,OUTPUT,"");
+	     }
+	  break;
+      case warnall:   
+	  sprintf(OUTPUT,"%s has permission %o\n",file,dstat->st_mode & 07777);
+	  CfLog(cferror,OUTPUT,"");
+	  sprintf(OUTPUT,"[should be %o]\n",newperm & 07777);
+	  CfLog(cferror,OUTPUT,"");
+	  break;
+	  
+      case fixplain:
+	  if (S_ISREG(dstat->st_mode))
+	     {
+	     if (! DONTDO)
+		{
+		if (chmod (file,newperm & 07777) == -1)
 		   {
-                   AddMultipleClasses(ptr->defines);
+		   sprintf(OUTPUT,"chmod failed on %s\n",file);
+		   CfLog(cferror,OUTPUT,"chmod");
+		   break;
 		   }
-                break;
-
-   default:     FatalError("cfengine: internal error CheckExistingFile(): illegal file action\n");
+		}
+	     
+	     sprintf(OUTPUT,"%s had permission %o, changed it to %o\n",
+		     file,dstat->st_mode & 07777,newperm & 07777);
+	     CfLog(cfinform,OUTPUT,"");
+	     
+	     if (ptr != NULL)
+		{
+		AddMultipleClasses(ptr->defines);
+		}
+	     }
+	  break;
+	  
+      case fixdirs:
+	  if (S_ISDIR(dstat->st_mode))
+	     {
+	     if (! DONTDO)
+		{
+		if (chmod (file,newperm & 07777) == -1)
+		   {
+		   sprintf(OUTPUT,"chmod failed on %s\n",file);
+		   CfLog(cferror,OUTPUT,"chmod");
+		   break;
+		   }
+		}
+	     
+	     sprintf(OUTPUT,"%s had permission %o, changed it to %o\n",
+		     file,dstat->st_mode & 07777,newperm & 07777);
+	     CfLog(cfinform,OUTPUT,"");
+	     
+	     if (ptr != NULL)
+		{
+		AddMultipleClasses(ptr->defines);
+		}
+	     }
+	  break;
+	  
+      case fixall:
+	  if (! DONTDO)
+	     {
+	     if (chmod (file,newperm & 07777) == -1)
+		{
+		sprintf(OUTPUT,"chmod failed on %s\n",file);
+		CfLog(cferror,OUTPUT,"chmod");
+		break;
+		}
+	     }
+	  
+	  sprintf(OUTPUT,"%s had permission %o, changed it to %o\n",
+		  file,dstat->st_mode & 07777,newperm & 07777);
+	  CfLog(cfinform,OUTPUT,"");
+	  
+	  if (ptr != NULL)
+	     {
+	     AddMultipleClasses(ptr->defines);
+	     }
+	  break;
+	  
+      case touch:
+	  if (! DONTDO)
+	     {
+	     if (chmod (file,newperm & 07777) == -1)
+		{
+		sprintf(OUTPUT,"chmod failed on %s\n",file);
+		CfLog(cferror,OUTPUT,"chmod");
+		break;
+		}
+	     utime (file,NULL);
+	     }
+	  if (ptr != NULL)
+	     {
+	     AddMultipleClasses(ptr->defines);
+	     }
+	  break;
+	  
+      default:     FatalError("cfengine: internal error CheckExistingFile(): illegal file action\n");
+      }
    }
 
+#if defined HAVE_CHFLAGS  /* BSD special flags */
+
+if (ptr != NULL)
+   {
+   newflags = (dstat->st_flags & CHFLAGS_MASK) ;
+   newflags |= ptr->plus_flags;
+   newflags &= ~(ptr->minus_flags);
+   
+   if ((newflags & CHFLAGS_MASK) == (dstat->st_flags & CHFLAGS_MASK))    /* file okay */
+      {
+      Debug("File okay, flags = %o, current = %o\n",(newflags & CHFLAGS_MASK),(dstat->st_flags & CHFLAGS_MASK));
+      }
+   else
+      {
+      Debug("Fixing %s, newflags = %o, flags = %o\n",file,(newflags & CHFLAGS_MASK),(dstat->st_flags & CHFLAGS_MASK));
+      
+      switch (action)
+	 {
+	 case linkchildren:
+	     
+	 case warnplain:
+	     if (S_ISREG(dstat->st_mode))
+		{
+		sprintf(OUTPUT,"%s has flags %o\n",file,dstat->st_flags & CHFLAGS_MASK);
+		CfLog(cferror,OUTPUT,"");
+		sprintf(OUTPUT,"[should be %o]\n",newflags & CHFLAGS_MASK);
+		CfLog(cferror,OUTPUT,"");
+		}
+	     break;
+	 case warndirs:
+	     if (S_ISDIR(dstat->st_mode))
+		{
+		sprintf(OUTPUT,"%s has flags %o\n",file,dstat->st_mode & CHFLAGS_MASK);
+		CfLog(cferror,OUTPUT,"");
+		sprintf(OUTPUT,"[should be %o]\n",newflags & CHFLAGS_MASK);
+		CfLog(cferror,OUTPUT,"");
+		}
+	     break;
+	 case warnall:
+	     sprintf(OUTPUT,"%s has flags %o\n",file,dstat->st_mode & CHFLAGS_MASK);
+	     CfLog(cferror,OUTPUT,"");
+	     sprintf(OUTPUT,"[should be %o]\n",newflags & CHFLAGS_MASK);
+	     CfLog(cferror,OUTPUT,"");
+	     break;
+	     
+	 case fixplain:
+	     
+	     if (S_ISREG(dstat->st_mode))
+		{
+		if (! DONTDO)
+		   {
+		   if (chflags (file,newflags & CHFLAGS_MASK) == -1)
+		      {
+		      sprintf(OUTPUT,"chflags failed on %s\n",file);
+		      CfLog(cferror,OUTPUT,"chflags");
+		      break;
+		      }
+		   }
+		
+		sprintf(OUTPUT,"%s had flags %o, changed it to %o\n",
+			file,dstat->st_flags & CHFLAGS_MASK,newflags & CHFLAGS_MASK);
+		CfLog(cfinform,OUTPUT,"");
+		
+		if (ptr != NULL)
+		   {
+		   AddMultipleClasses(ptr->defines);
+		   }
+		}
+	     break;
+	     
+	 case fixdirs:
+	     if (S_ISDIR(dstat->st_mode))
+		{
+		if (! DONTDO)
+		   {
+		   if (chflags (file,newflags & CHFLAGS_MASK) == -1)
+		      {
+		      sprintf(OUTPUT,"chflags failed on %s\n",file);
+		      CfLog(cferror,OUTPUT,"chflags");
+		      break;
+		      }
+		   }
+		
+		sprintf(OUTPUT,"%s had flags %o, changed it to %o\n",
+			file,dstat->st_flags & CHFLAGS_MASK,newflags & CHFLAGS_MASK);
+		CfLog(cfinform,OUTPUT,"");
+		
+		if (ptr != NULL)
+		   {
+		   AddMultipleClasses(ptr->defines);
+		   }
+		}
+	     break;
+	     
+      case fixall:
+	  if (! DONTDO)
+	     {
+	     if (chflags (file,newflags & CHFLAGS_MASK) == -1)
+		{
+		sprintf(OUTPUT,"chflags failed on %s\n",file);
+		CfLog(cferror,OUTPUT,"chflags");
+		break;
+		}
+	     }
+	  
+	  sprintf(OUTPUT,"%s had flags %o, changed it to %o\n",
+		  file,dstat->st_flags & CHFLAGS_MASK,newflags & CHFLAGS_MASK);
+	  CfLog(cfinform,OUTPUT,"");
+	  
+	  if (ptr != NULL)
+	     {
+	     AddMultipleClasses(ptr->defines);
+	     }
+	  break;
+	  
+      case touch:
+	  if (! DONTDO)
+	     {
+	     if (chflags (file,newflags & CHFLAGS_MASK) == -1)
+		{
+		sprintf(OUTPUT,"chflags failed on %s\n",file);
+		CfLog(cferror,OUTPUT,"chflags");
+		break;
+		}
+	     utime (file,NULL);
+	     }
+	  if (ptr != NULL)
+	     {
+	     AddMultipleClasses(ptr->defines);
+	     }
+	  break;
+	  
+	 default:     FatalError("cfengine: internal error CheckExistingFile(): illegal file action\n");
+	 }
+      }
+   }
+#endif
+
+ 
 Debug("CheckExistingFile(Done)\n"); 
 }
 
@@ -604,8 +770,8 @@ struct stat *statbuf;
   struct UidList *ulp;
   struct GidList *glp;
   short uidmatch = false, gidmatch = false;
-  int uid = -1; 
-  int gid = -1;
+  uid_t uid = (uid_t)-1; 
+  gid_t gid = (gid_t)-1;
 
 Debug("CheckOwner: %d\n",statbuf->st_uid);
   
@@ -613,7 +779,7 @@ for (ulp = uidlist; ulp != NULL; ulp=ulp->next)
    {
    Debug(" uid %d\n",ulp->uid);
 
-   if (ulp->uid == -1 || statbuf->st_uid == ulp->uid)   /* -1 matches anything */
+   if (ulp->uid == (uid_t) -1 || statbuf->st_uid == ulp->uid)   /* -1 matches anything */
       {
       uid = ulp->uid;
       uidmatch = true;
@@ -623,7 +789,7 @@ for (ulp = uidlist; ulp != NULL; ulp=ulp->next)
 
 for (glp = gidlist; glp != NULL; glp=glp->next)
    {
-   if (glp->gid == -1 || statbuf->st_gid == glp->gid)  /* -1 matches anything */
+   if (glp->gid == (gid_t) -1 || statbuf->st_gid == glp->gid)  /* -1 matches anything */
       {
       gid = glp->gid;
       gidmatch = true;
@@ -668,19 +834,19 @@ else
       case touch:
                   if (VERBOSE || DEBUG || D2)
                      {
-                     if (uid == -1 && gid == -1)
+                     if (uid == (uid_t) -1 && gid == (gid_t) -1)
                         {
                         printf("%s:   touching %s\n",VPREFIX,file);
                         }
                      else
                         {
-                        if (uid != -1)
+                        if (uid != (uid_t) -1)
                            {
                            sprintf(OUTPUT,"  (Change owner to uid %d if possible)\n",uid);
                            CfLog(cfinform,OUTPUT,"");
                            }
 
-                        if (gid != -1)
+                        if (gid != (gid_t) -1)
                            {
                            sprintf(OUTPUT,"  (Change group to gid %d if possible)\n",gid);
                            CfLog(cfinform,OUTPUT,"");
