@@ -33,19 +33,6 @@
 #include "cf.defs.h"
 #include "cf.extern.h"
 
-/*********************************************************************/
-/* Files to be ignored when parsing directories                      */
-/*********************************************************************/
-
-char *VSKIPFILES[] =
-   {
-   ".",
-   "..",
-   "lost+found",
-   ".cfengine.rm",
-   NULL
-   };
-
 /*********************************************************************/
 /* Level 1                                                           */
 /*********************************************************************/
@@ -327,170 +314,6 @@ str[strlen(str)-1] = '\0';
 }
 
 
-/*********************************************************************/
-
-SensibleFile(nodename,path)
-
-char *nodename, *path;
-
-{ int i, suspicious = true;
-  struct stat statbuf; 
-  unsigned char *sp, newname[bufsize];
-
-Debug("SensibleFile(%s,%s)\n",path,nodename);
-
-if (strlen(nodename) < 1)
-   {
-   sprintf(OUTPUT,"Empty (null) filename detected in %s\n",path);
-   CfLog(cferror,OUTPUT,"");
-   return false;
-   }
-
-if (IsItemIn(SUSPICIOUSLIST,nodename))
-   {
-   sprintf(OUTPUT,"Suspicious file %s found in %s\n",nodename,path);
-   CfLog(cferror,OUTPUT,"");
-   return false;
-   }
-
-if (strcmp(nodename,"...") == 0 && strcmp(path,"/") == 0)
-   {
-   Verbose("DFS cell node detected in /...\n");
-   return true;
-   }
-  
-for (i = 0; VSKIPFILES[i] != NULL; i++)
-   {
-   if (strcmp(nodename,VSKIPFILES[i]) == 0)
-      {
-      Debug("Filename is classified as ignorable\n");
-      return false;
-      }
-   }
-
-if ((strcmp("[",nodename) == 0) && (strcmp("/usr/bin",path) == 0))
-   {
-   if (VSYSTEMHARDCLASS == linuxx)
-      {
-      return true;
-      }
-   }
-
-suspicious = true;
-
-for (sp = nodename; *sp != '\0'; sp++)
-   {
-   if ((*sp > 31) && (*sp < 127))
-      {
-      suspicious = false;
-      break;
-      }
-   }
-
-strcpy(VBUFF,path);
-AddSlash(VBUFF);
-strcat(VBUFF,nodename); 
-
-if (suspicious && NONALPHAFILES)
-   {
-   sprintf(OUTPUT,"Suspicious filename %s in %s has no alphanumeric content (security)",nodename,path);
-   CfLog(cfsilent,OUTPUT,"");
-   strcpy(newname,VBUFF);
-
-   for (sp = newname+strlen(path); *sp != '\0'; sp++)
-      {
-      if ((*sp > 126) || (*sp < 32))
-	 {
-	 *sp = 50 + (*sp / 50);  /* Create a visible ASCII interpretation */
-	 }
-      }
-
-   strcat(newname,".cf-nonalpha");
-   
-   sprintf(OUTPUT,"Disabling file %s to %s",VBUFF,newname);
-   CfLog(cfsilent,OUTPUT,"");
-   
-   if (rename(VBUFF,newname) == -1)
-      {
-      CfLog(cfverbose,"Rename failed - foreign filesystem?","rename");
-      }
-   if (chmod(newname,0644) == -1)
-      {
-      CfLog(cfverbose,"Mode change failed - foreign filesystem?","chmod");
-      }
-   return false;
-   }
-
-if (strstr(nodename,".") && (EXTENSIONLIST != NULL))
-   {
-   if (lstat(VBUFF,&statbuf) == -1)
-      {
-      sprintf(OUTPUT,"Couldn't examine %s - foreign filesystem?",VBUFF);
-      CfLog(cfverbose,OUTPUT,"lstat");
-      return true;
-      }
-
-   if (S_ISDIR(statbuf.st_mode))
-      {
-      for (sp = nodename+strlen(nodename)-1; *sp != '.'; sp--)
-	 {
-	 }
-
-      if ((char *)sp != nodename) /* Don't get .dir */
-	 {
-	 sp++; /* Find file extension, look for known plain files  */
-
-	 if (IsItemIn(EXTENSIONLIST,sp))
-	    {
-	    sprintf(OUTPUT,"Suspicious directory %s in %s looks like plain file with extension .%s",nodename,path,sp);
-	    CfLog(cfsilent,OUTPUT,"");
-	    return false;
-	    }
-	 }
-      }
-   }
-
-for (sp = nodename; *sp != '\0'; sp++) /* Check for files like ".. ." */
-   {
-   if ((*sp != '.') && ! isspace(*sp))
-      {
-      Debug("Filename is not suspicious\n");
-      suspicious = false;
-      return true;
-      }
-   }
-
-sprintf(OUTPUT,"Suspicous looking file object \"%s\" masquerading as hidden file in %s\n",nodename,path);
-CfLog(cfsilent,OUTPUT,"");
-Debug("Filename looks suspicious\n"); 
-
-
-if (EXTENSIONLIST == NULL)
-   {
-   if (lstat(VBUFF,&statbuf) == -1)
-      {
-      sprintf(OUTPUT,"Couldn't stat %s",VBUFF);
-      CfLog(cfsilent,OUTPUT,"lstat");
-      return true;
-      }
-   }
-
-if (S_ISLNK(statbuf.st_mode))
-   {
-   sprintf(OUTPUT,"   %s is a symbolic link\n",nodename);
-   CfLog(cfsilent,OUTPUT,"");
-   }
-else if (S_ISDIR(statbuf.st_mode))
-   {
-   sprintf(OUTPUT,"   %s is a directory\n",nodename);
-   CfLog(cfsilent,OUTPUT,"");
-   }
-
-sprintf(OUTPUT,"%s has size %d and full mode %o\n",nodename,statbuf.st_size,statbuf.st_mode);
-CfLog(cfsilent,OUTPUT,"");
- 
-return true;
-}
 
 /*********************************************************************/
 
