@@ -133,6 +133,8 @@ int CHECKSUMUPDATES = false;
   PROTECTED int VDEFAULTIFELAPSED = 0;    /* minutes */
   PROTECTED int VDEFAULTEXPIREAFTER = 120;
 
+  PUBLIC int SD = cf_not_connected; /* socket */
+
 /*******************************************************************/
  /* Data structures - root pointers                                 */
  /*******************************************************************/
@@ -147,6 +149,7 @@ int CHECKSUMUPDATES = false;
   PROTECTED  struct Item *VCPLNPARSE = NULL;
   PROTECTED  struct Item *VINCLUDEPARSE = NULL;
   PROTECTED  struct Item *VIGNOREPARSE = NULL;
+  PROTECTED  struct Item *VSERVERLIST = NULL;
 
   PROTECTED  struct Item *VHEAP = NULL;      /* Points to the base of the attribute heap */
   PROTECTED  struct Item *VNEGHEAP = NULL;
@@ -467,6 +470,7 @@ int CHECKSUMUPDATES = false;
   PRIVATE char LINKTYPE = 's';
   PRIVATE char AGETYPE = 'a';
   PRIVATE char COPYTYPE = 't';
+  PRIVATE char DEFAULTCOPYTYPE = 't';
   PRIVATE char REPOSCHAR = '_';
   PRIVATE char LISTSEPARATOR = ':';
   PRIVATE char LINKDIRS = 'k';
@@ -510,10 +514,12 @@ int CHECKSUMUPDATES = false;
   PRIVATE int VAGE;
   PRIVATE int VTIMEOUT=0;
 
-
   PRIVATE mode_t PLUSMASK;
   PRIVATE mode_t MINUSMASK;
 
+  PRIVATE u_long PLUSFLAG;
+  PRIVATE u_long MINUSFLAG;
+
  /* Parsing flags etc */
 
   PUBLIC enum actions ACTION = none;
@@ -527,6 +533,7 @@ int CHECKSUMUPDATES = false;
   PRIVATE flag HAVE_RESTART = false;
   PRIVATE flag ACTIONPENDING = false;
   PRIVATE flag HOMECOPY=false;
+  PRIVATE short SECURE=false;
 
   PRIVATE char *COMMATTRIBUTES[] =
      {
@@ -571,6 +578,9 @@ int CHECKSUMUPDATES = false;
      "deletefstab",
      "stealth",
      "checksum",
+     "flags",
+     "secure",
+     "root",
      NULL
      };
 
@@ -742,6 +752,10 @@ int CHECKSUMUPDATES = false;
   PRIVATE char CFSERVER[maxvarsize];
   PRIVATE int  CF_TIMEOUT = 10;
 
+  PRIVATE int  CFSIGNATURE;
+  PRIVATE char CFDES1[8];
+  PRIVATE char CFDES2[8];
+  PRIVATE char CFDES3[8];
 
 /*******************************************************************/
 /*                                                                 */
