@@ -66,26 +66,27 @@
 /*******************************************************************/
 
 #define INET
-
-/* IRIX makes the routing stuff obsolete unless we do this */
-#undef sgi
-
 #include "cf.defs.h"
 #include "cf.extern.h"
 
+#if !defined(NT) && !defined(IRIX)
+
+/* IRIX makes the routing stuff obsolete unless we do this */
+# undef sgi
+
 struct ifreq IFR;
 
 char VNUMBROADCAST[256];
 
-#define cfproto 0
+# define cfproto 0
 
-#ifndef IPPROTO_IP     /* Old boxes, hpux 7 etc */
-# define IPPROTO_IP 0
-#endif
+# ifndef IPPROTO_IP     /* Old boxes, hpux 7 etc */
+#  define IPPROTO_IP 0
+# endif
 
-#ifndef SIOCSIFBRDADDR
-# define SIOCSIFBRDADDR  SIOCGIFBRDADDR
-#endif
+# ifndef SIOCSIFBRDADDR
+#  define SIOCSIFBRDADDR  SIOCGIFBRDADDR
+# endif
 
 /*******************************************************************/
 
@@ -95,9 +96,10 @@ char *vifdev,*vnetmask, *vbroadcast;
 
 { int sk, flags, metric, isnotsane = false;
 
+ 
 Verbose("Assumed interface name: %s %s %s\n",vifdev,vnetmask,vbroadcast);
 
- if (getuid() != 0)                            
+if (getuid() != 0)                            
    {
    printf("%s: Only root can configure the net interface.\n",VPREFIX);
    return;
@@ -229,7 +231,7 @@ GetBroadcastAddr(inet_ntoa(inaddr),vifdev,vnetmask,vbroadcast);
 
 if (strcmp(VBUFF,VNUMBROADCAST) != 0)
    {
-   sprintf(OUTPUT,"Broadcast address was %s (should be %s)\n",VBUFF,vbroadcast);
+   sprintf(OUTPUT,"Broadcast address was %s (should be bit-type %s)\n",VBUFF,vbroadcast);
    CfLog(cferror,OUTPUT,"");
    insane = true;
    }
@@ -261,13 +263,13 @@ char *vifdev,*vnetmask, *vbroadcast;
 
 /* set netmask */
 
-Verbose("Resetting interface");
+Verbose("Resetting interface...\n");
 strcpy(IFR.ifr_name,vifdev);
 netmask.sin_addr.s_addr = inet_network(vnetmask);
 IFR.ifr_addr = *((struct sockaddr *) &netmask);
 sin = (struct sockaddr_in *) &IFR.ifr_addr;
 
-if (ioctl(sk,SIOCSIFNETMASK, (caddr_t) &IFR)) 
+if (ioctl(sk,SIOCSIFNETMASK, (caddr_t) &IFR) < 0) 
    {
    CfLog(cferror,"Permission to reconfigure netmask denied.\n","ioctl");
    }
@@ -284,6 +286,8 @@ broadcast.sin_addr.s_addr = inet_addr(VNUMBROADCAST);
 IFR.ifr_addr = *((struct sockaddr *) &broadcast);
 sin = (struct sockaddr_in *) &IFR.ifr_addr;
 
+Verbose("Trying to set broad to %s = %s\n",VNUMBROADCAST,inet_ntoa(sin->sin_addr));
+ 
 if (ioctl(sk,SIOCSIFBRDADDR, (caddr_t) &IFR) == -1) 
    {
    CfLog(cferror,"Permission to reconfigure broadcast denied.\n","ioctl");
@@ -356,11 +360,11 @@ SetDefaultRoute()
 { int sk, defaultokay = 1;
   struct sockaddr_in sindst,singw;
 
-#ifdef HAVE_ORTENTRY
-  struct ortentry route;
-#elif HAVE_RTENTRY
-  struct rtentry route;
-#endif
+# ifdef HAVE_ORTENTRY
+   struct ortentry route;
+# else if HAVE_RTENTRY
+   struct rtentry route;
+# endif
 
   FILE *pp;
 
@@ -394,7 +398,7 @@ while (!feof(pp))
       {
       if (strstr(VBUFF,VDEFAULTROUTE))
          {
-	 Debug("cfengine: default route is already set to %s\n",VDEFAULTROUTE);
+	 Verbose("cfengine: default route is already set to %s\n",VDEFAULTROUTE);
          defaultokay = 1;
          break;
          }
@@ -415,7 +419,7 @@ if (defaultokay)
    return;
    }
 
-#if defined HAVE_RTENTRY || defined HAVE_ORTENTRY
+# if defined HAVE_RTENTRY || defined HAVE_ORTENTRY
  
 if ((sk = socket(AF_INET,SOCK_RAW,cfproto)) == -1)
    {
@@ -453,11 +457,29 @@ if (! DONTDO)
    CfLog(cferror,OUTPUT,"");
    }
 
-#else if
+# else if
 
 /* Socket routing - don't really know how to do this yet */ 
 
 Verbose("Sorry don't know how to do routing on this platform\n");
  
-#endif
+# endif
 }
+
+#else /* NT */
+
+IfConf (vifdev,vnetmask,vbroadcast)
+
+char *vifdev,*vnetmask, *vbroadcast;
+
+{
+Verbose("Network configuration is not implemented on this OS\n");
+}
+
+SetDefaultRoute()
+
+{
+Verbose("Setting default route is not implemented on this OS\n"); 
+}
+
+#endif
