@@ -93,7 +93,7 @@ if ((dirh = cfopendir(from,ip)) == NULL)
 
 for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
    {
-   if (!SensibleFile(dirp->d_name,from))
+   if (!SensibleFile(dirp->d_name,from,ip))
       {
       continue;
       }
@@ -108,11 +108,6 @@ for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
       continue;
       }
 
-   if (IgnoredOrExcluded(image,dirp->d_name,ip->inclusions,ip->exclusions))
-      {
-      continue;
-      }
-   
    strcpy(newfrom,from);                                   /* Assemble pathname */
    AddSlash(newfrom);
    strcpy(newto,to);
@@ -153,18 +148,23 @@ for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
          }
       }
 
+   if (!S_ISDIR(statbuf.st_mode) && IgnoredOrExcluded(image,dirp->d_name,ip->inclusions,ip->exclusions))
+      {
+      continue;
+      }
+   
    if (S_ISDIR(statbuf.st_mode))
       {	
       bzero(&deststatbuf,sizeof(struct stat));
       save_uid = (ip->uid)->uid;
       save_gid = (ip->gid)->gid;
       
-      if ((ip->uid)->uid == -1)          /* Preserve uid and gid  */
+      if ((ip->uid)->uid == (uid_t)-1)          /* Preserve uid and gid  */
 	 {
 	 (ip->uid)->uid = statbuf.st_uid;
 	 }
       
-      if ((ip->gid)->gid == -1)
+      if ((ip->gid)->gid == (gid_t)-1)
 	 {
 	 (ip->gid)->gid = statbuf.st_gid;
 	 }
@@ -187,10 +187,7 @@ for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
       }
    }
 
-
- /* If we purge with no authentication we wipe out EVERYTHING */ 
-
-if (AUTHENTICATED && (ip->purge == 'y'))
+if (ip->purge == 'y')
    {
    PurgeFiles(namecache,to);
    DeleteItemList(namecache);
@@ -247,7 +244,7 @@ for (itp = VMOUNTLIST; itp != NULL; itp=itp->next)
 
    for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
       {
-      if (!SensibleFile(dirp->d_name,itp->name))
+      if (!SensibleFile(dirp->d_name,itp->name,ip))
          {
          continue;
          }
@@ -270,7 +267,7 @@ for (itp = VMOUNTLIST; itp != NULL; itp=itp->next)
 
       for (dirp2 = cfreaddir(dirh2,ip); dirp2 != NULL; dirp2 = cfreaddir(dirh2,ip))
          {
-         if (!SensibleFile(dirp2->d_name,homedir))
+         if (!SensibleFile(dirp2->d_name,homedir,ip))
             {
             continue;
             }
@@ -283,7 +280,16 @@ for (itp = VMOUNTLIST; itp != NULL; itp=itp->next)
          if (strlen(ip->destination) > 4)
             {
             AddSlash(dest);
-            strcat(dest,ReadLastNode(ip->destination));
+	    if (strlen(ip->destination) < 6)
+	       {
+	       sprintf(OUTPUT,"Error in home/copy to %s",ip->destination);
+	       CfLog(cferror,OUTPUT,"");
+	       return;
+	       }
+	    else
+	       {
+	       strcat(dest,(ip->destination)+strlen("home/"));
+	       }
             }
 
          if (request_uid == -1)
@@ -355,12 +361,12 @@ if (sourcestatbuf.st_nlink > 1)    /* Preserve hard link structure when copying
 save_uid = (ip->uid)->uid;
 save_gid = (ip->gid)->gid;
 
-if ((ip->uid)->uid == -1)          /* Preserve uid and gid  */
+if ((ip->uid)->uid == (uid_t)-1)          /* Preserve uid and gid  */
    {
    (ip->uid)->uid = sourcestatbuf.st_uid;
    }
 
-if ((ip->gid)->gid == -1)
+if ((ip->gid)->gid == (gid_t)-1)
    {
    (ip->gid)->gid = sourcestatbuf.st_gid;
    }
@@ -388,7 +394,7 @@ if (S_ISDIR(sourcestatbuf.st_mode))
    
    for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
       {
-      if (!SensibleFile(dirp->d_name,sourcedir))
+      if (!SensibleFile(dirp->d_name,sourcedir,ip))
          {
          continue;
          }
@@ -451,7 +457,15 @@ char *directory;
   char filename[bufsize];
 
 Debug("PurgeFiles(%s)\n",directory);
+
+ /* If we purge with no authentication we wipe out EVERYTHING */ 
  
+ if (!AUTHENTICATED)
+    {
+    Verbose("Not purging %s - no verified contact with server\n",directory);
+    return;
+    }
+
  if ((dirh = opendir(directory)) == NULL)
     {
     sprintf(OUTPUT,"Can't open directory %s\n",directory);
@@ -461,7 +475,7 @@ Debug("PurgeFiles(%s)\n",directory);
 
  for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
     {
-    if (!SensibleFile(dirp->d_name,directory))
+    if (!SensibleFile(dirp->d_name,directory,NULL))
        {
        continue;
        }
@@ -801,7 +815,7 @@ else
 		      }	     
 	           else
 	              {
-		      Verbose("%s: Checksum comparison replaced by ctime: files not regular\n",VPREFIX);
+		      Verbose("%s: Byte comparison replaced by ctime: files not regular\n",VPREFIX);
 		      Verbose("%s: %s -> %s\n",VPREFIX,sourcefile,destfile);
 		      ok_to_copy = (deststatbuf.st_ctime < sourcestatbuf.st_ctime)||(deststatbuf.st_mtime < sourcestatbuf.st_mtime);
 	              }
@@ -813,6 +827,16 @@ else
                       return;
                       }
 	           break;
+
+	 case 'm': ok_to_copy = (deststatbuf.st_mtime < sourcestatbuf.st_mtime);
+	     
+                   if (ok_to_copy && strcmp(ip->action,"warn") == 0)
+                      { 
+                      Verbose("%s: Image file %s out of date\n",VPREFIX,destfile);
+                      Verbose("%s: (should be copy of %s)\n", VPREFIX,sourcefile);
+                      return;
+                      }
+	           break;
 		   
          default:  ok_to_copy = (deststatbuf.st_ctime < sourcestatbuf.st_ctime)||(deststatbuf.st_mtime < sourcestatbuf.st_mtime);
 	     
@@ -926,6 +950,13 @@ struct Image *ip;
 
 { int res;
 
+if (ip == NULL)
+   {
+   res = stat(file,buf);
+   /* CheckForHoles(buf,ip); Don't need this if ip is NULL */
+   return res;
+   }
+
 if (strcmp(ip->server,"localhost") == 0)
    {
    res = stat(file,buf);
@@ -950,6 +981,13 @@ struct Image *ip;
 
 { int res;
 
+if (ip == NULL)
+   {
+   res = lstat(file,buf);
+   /* CheckForHoles(buf,ip); Don't need this if ip is NULL */
+   return res;
+   }
+
 if (strcmp(ip->server,"localhost") == 0)
    {
    res = lstat(file,buf);
@@ -1036,7 +1074,7 @@ if (strcmp(ip->server,"localhost") == 0)
       }
    }
 else
-   { 
+   {
    return cf_ropendir(name,ip);
    }
 }
@@ -1155,7 +1193,7 @@ strcat(new,CF_NEW);
 
 if (remote)
    {
-   if (!CopyRegNet(source,new,ip))
+   if (!CopyRegNet(source,new,ip,sstat.st_size))
       {
       return false;
       }
@@ -1167,7 +1205,7 @@ else
       return false;
       }
 
-   if (ip->stealth)
+   if (ip->stealth == 't')
       {
 #ifdef HAVE_UTIME_H
       timebuf.actime = sstat.st_atime;
@@ -1177,7 +1215,7 @@ else
       }
    }
 
-Debug2("CopyReg succeeded in copying to .new\n");
+Debug("CopyReg succeeded in copying to %s to %s\n",source,new);
 
 if (IMAGEBACKUP)
    {
@@ -1197,9 +1235,9 @@ if (IMAGEBACKUP)
 
 stat(new,&dstat);
 
-if ((dstat.st_size == 0) && (sstat.st_size != 0))
+if (dstat.st_size != sstat.st_size)
    {
-   sprintf(OUTPUT,"WARNING: new file was empty %s (old one wasn't) aborting copy\n",new);
+   sprintf(OUTPUT,"WARNING: new file %s seems to have been corrupted in transit, aborting!\n",new);
    CfLog(cfverbose,OUTPUT,"");
    if (rename(backup,dest) == -1)
       {
@@ -1208,9 +1246,24 @@ if ((dstat.st_size == 0) && (sstat.st_size != 0))
    return false;
    }
 
+if (ip->secure)
+   {
+   Debug("Final verification of transmission...\n");
+   if (CompareMD5CheckSums(source,new,ip,&sstat,&dstat))
+      {
+      sprintf(OUTPUT,"WARNING: new file %s seems to have been corrupted in transit, aborting!\n",new);
+      CfLog(cfverbose,OUTPUT,"");
+      if (rename(backup,dest) == -1)
+	 {
+	 /* ignore */
+	 }
+      return false;
+      }
+   }
+
 if (rename(new,dest) == -1)
    {
-   sprintf(OUTPUT,"SERIOUS PROBLEM! Could not install copy file as %s\n",dest);
+   sprintf(OUTPUT,"Problem: could not install copy file as %s, directory in the way?\n",dest);
    CfLog(cferror,OUTPUT,"rename");
    rename(backup,dest);
    return false;
@@ -1242,7 +1295,7 @@ for (sp = ip->defines; *sp != '\0'; sp++)
 
    sscanf(sp,"%[^,:.]",currentitem);
 
-   sp += strlen(currentitem);
+   sp += strlen(currentitem)-1;
 
    AddClassToHeap(currentitem);
    }
