@@ -1,6 +1,6 @@
 /* cfengine for GNU
  
-        Copyright (C) 1995
+        Copyright (C) 1995,1999
         Free Software Foundation, Inc.
  
    This file is part of GNU cfengine - written and maintained 
@@ -228,47 +228,40 @@ struct Item *list;
 char *string;
 
 { struct Item *ip;
-  char fastmap[(1 << BYTEWIDTH)];
-  struct re_pattern_buffer buf;
-  struct re_registers regs;
-  char *err;
-  int match;
+  regex_t rx;
+  regmatch_t pmatch;
+  size_t nmatch = 1;
+int ret;
 
-buf.allocated = 0;
-buf.buffer = NULL;
-buf.fastmap = fastmap;
-buf.translate = 0;
+printf("Looking for %s\n",string);
 
-if ((err = re_compile_pattern(string,strlen(string),&buf)) != 0)
+if (regcomp(&rx,string, REG_EXTENDED) != 0)
    {
    sprintf(OUTPUT,"Regular expression error for %s\n",string);
-   CfLog(cferror,OUTPUT,"");
-   sprintf("%s\n",err);
-   CfLog(cferror,OUTPUT,"");
+   CfLog(cferror,OUTPUT,"regcomp");
    return NULL;
    }
 
-re_compile_fastmap (&buf);
-
 for (ip = list; (ip != NULL); ip=ip->next)
    {
    if (EDABORTMODE && ItemMatchesRegEx(ip->name,VEDITABORT))
       {
       Verbose("Aborting search, regex %s matches line\n",VEDITABORT);
+      regfree(&rx);
       return NULL;
       }
       
-   if ((match = re_match(&buf,ip->name,strlen(ip->name),0,&regs)) == strlen(ip->name))
+   if (regexec(&rx,ip->name,1,&pmatch,0) == 0)
       {
-      return ip;
-      }
-   else if (match < -1)
-      {
-      sprintf(OUTPUT,"Regular expression internal error [%s]\n",string);
-      CfLog(cferror,OUTPUT,"");
+      if ((pmatch.rm_so == 0) && (pmatch.rm_eo == strlen(ip->name)))
+	 {
+         regfree(&rx);
+	 return ip;
+	 }
       }
    }
 
+regfree(&rx);
 return NULL;
 }
 
@@ -306,54 +299,45 @@ struct Item *list;
 char *string;
 
 { struct Item *ip;
-  char fastmap[(1 << BYTEWIDTH)];
-  struct re_pattern_buffer buf;
-  struct re_registers regs;
-  int match, line = CURRENTLINENUMBER;
-  char *err;
+  regex_t rx;
+  regmatch_t pmatch;
+  size_t nmatch = 1;
+  int line = CURRENTLINENUMBER;
 
-Debug2("LocateItemMatchingRexExp(%s,%s)\n",list->name,string);
+Debug("LocateItemMatchingRexExp(%s,%s)\n",list->name,string);
   
-buf.allocated = 0;
-buf.buffer = NULL;
-buf.fastmap = fastmap;
-buf.translate = 0;
-
-if ((err = re_compile_pattern(string,strlen(string),&buf)) != 0)
+if (regcomp(&rx,string, REG_EXTENDED) != 0)
    {
    sprintf(OUTPUT,"Regular expression error for %s\n",string);
-   CfLog(cferror,OUTPUT,"");
-   sprintf(OUTPUT,"%s\n",err);
-   CfLog(cferror,OUTPUT,"");
+   CfLog(cferror,OUTPUT,"regcomp");
    return NULL;
    }
 
-re_compile_fastmap (&buf);
-
 for (ip = list; (ip != NULL); ip=ip->next, line++)
    {
    if (EDABORTMODE && ItemMatchesRegEx(ip->name,VEDITABORT))
       {
       Verbose("Aborting search, regex %s matches line\n",VEDITABORT);
+      regfree(&rx);
       return NULL;
       }
       
-   if ((match = re_match(&buf,ip->name,strlen(ip->name),0,&regs)) == strlen(ip->name))
+   if (regexec(&rx,ip->name,1,&pmatch,0) == 0)
       {
-      EditVerbose("Edit: Search ended at line %d\n",line);
-      EditVerbose("Edit: (Found %s)\n",ip->name);
-      CURRENTLINENUMBER = line;
-      CURRENTLINEPTR = ip;
-      return ip;
-      }
-   else if (match < -1)
-      {
-      sprintf(OUTPUT,"Regular expression internal error [%s]\n",string);
-      CfLog(cferror,OUTPUT,"");
+      if ((pmatch.rm_so == 0) && (pmatch.rm_eo == strlen(ip->name)))
+	 {
+	 EditVerbose("Edit: Search ended at line %d\n",line);
+	 EditVerbose("Edit: (Found %s)\n",ip->name);
+	 CURRENTLINENUMBER = line;
+	 CURRENTLINEPTR = ip;
+         regfree(&rx);
+	 return ip;
+	 }
       }
    }
 
 EditVerbose("Edit: Search for %s failed. Current line still %d\n",string,CURRENTLINENUMBER);
+regfree(&rx);
 return NULL;
 }
 
@@ -365,52 +349,40 @@ struct Item *list;
 char *string;
 
 { struct Item *ip;
-  char fastmap[(1 << BYTEWIDTH)];
-  struct re_pattern_buffer buf;
-  struct re_registers regs;
-  int match, line = 1;
-  char *err;
+  regex_t rx;
+  regmatch_t pmatch;
+  size_t nmatch = 1;
+  int line = 1;
 
-buf.allocated = 0;
-buf.buffer = NULL;
-buf.fastmap = fastmap;
-buf.translate = 0;
-
-if ((err = re_compile_pattern(string,strlen(string),&buf)) != 0)
+if (regcomp(&rx,string, REG_EXTENDED) != 0)
    {
    sprintf(OUTPUT,"Regular expression error for %s\n",string);
-   CfLog(cferror,OUTPUT,"");
-   sprintf(OUTPUT,"%s\n",err);
-   CfLog(cferror,OUTPUT,"");
+   CfLog(cferror,OUTPUT,"regcomp");
    return NULL;
    }
 
-re_compile_fastmap (&buf);
-
 for (ip = list; (ip != NULL); ip=ip->next)
    {
    if (EDABORTMODE && ItemMatchesRegEx(ip->name,VEDITABORT))
       {
       Verbose("Aborting search, regex %s matches line\n",VEDITABORT);
+      regfree(&rx);
       return NULL;
       }
       
-   if ((match = re_search(&buf,ip->name,strlen(ip->name),0,strlen(ip->name),&regs)) >= 0)
+   if (regexec(&rx,ip->name,1,&pmatch,0) == 0)
       {
       EditVerbose("Search ended at line %d\n",line);
       CURRENTLINENUMBER = line;
       CURRENTLINEPTR = ip;
+      regfree(&rx);
       return ip;
       }
-   else if (match < 0)
-      {
-      sprintf(OUTPUT,"ERROR in regular expression [%s]!\n",string);
-      CfLog(cferror,OUTPUT,"");
-      }
    }
 
 EditVerbose("Search for %s failed. Current line still %d\n",string,CURRENTLINENUMBER);
 
+regfree(&rx);
 return NULL;
 }
 
@@ -424,11 +396,10 @@ struct Item **filestart;
 char *string;
 
 { struct Item *ip, *ip_prev, *ip_end = NULL;
-  char fastmap[(1 << BYTEWIDTH)];
-  struct re_pattern_buffer buf;
-  struct re_registers regs;
-  int match, linefound = false, done;
-  char *err;
+  int linefound = false, done;
+  regex_t rx;
+  regmatch_t pmatch;
+  size_t nmatch = 1;
 
 Debug2("DeleteToRegExp(list,%s)\n",string);
 
@@ -438,17 +409,10 @@ if (CURRENTLINEPTR == NULL)  /* Shouldn't happen */
    return true;
    }
 
-buf.allocated = 0;
-buf.buffer = NULL;
-buf.fastmap = fastmap;
-buf.translate = 0;
-
-if ((err = re_compile_pattern(string,strlen(string),&buf)) != 0)
+if (regcomp(&rx,string, REG_EXTENDED) != 0)
    {
    sprintf(OUTPUT,"Regular expression error for %s\n",string);
-   CfLog(cferror,OUTPUT,"");
-   sprintf(OUTPUT,"%s\n",err);
-   CfLog(cferror,OUTPUT,"");
+   CfLog(cferror,OUTPUT,"regcomp");
    return false;
    }
 
@@ -457,19 +421,18 @@ for (ip = CURRENTLINEPTR; (ip != NULL); ip=ip->next)
    if (EDABORTMODE && ItemMatchesRegEx(ip->name,VEDITABORT))
       {
       Verbose("Aborting search, regex %s matches line\n",VEDITABORT);
+      regfree(&rx);
       return false;
       }
       
-   if ((match = re_match(&buf,ip->name,strlen(ip->name),0,&regs)) == strlen(ip->name))
+   if (regexec(&rx,ip->name,1,&pmatch,0) == 0)
       {
-      linefound = true;
-      ip_end = ip;
-      break;
-      }
-   else if (match < -1)
-      {
-      sprintf(OUTPUT,"Regular expression internal error [%s]\n",string);
-      CfLog(cferror,OUTPUT,"");
+      if ((pmatch.rm_so == 0) && (pmatch.rm_eo == strlen(ip->name)))
+	 {
+	 linefound = true;
+	 ip_end = ip;
+	 break;
+	 }
       }
    }
 
@@ -495,6 +458,7 @@ for (ip = CURRENTLINEPTR; ip != NULL; ip = CURRENTLINEPTR)
    if (EDABORTMODE && ItemMatchesRegEx(ip->name,VEDITABORT))
       {
       Verbose("Aborting search, regex %s matches line\n",VEDITABORT);
+      regfree(&rx);
       return false;
       }
 
@@ -520,6 +484,7 @@ for (ip = CURRENTLINEPTR; ip != NULL; ip = CURRENTLINEPTR)
       }
    }
 
+regfree(&rx);
 return true;
 }
 
@@ -591,21 +556,14 @@ struct Item **list;
 char *string;
 
 { struct Item *ip, *next, *last;
-  char fastmap[(1 << BYTEWIDTH)];
-  struct re_pattern_buffer buf;
-  struct re_registers regs;
-  int match;
-  char *err;
+  regex_t rx,rxcache;
+  regmatch_t pmatch;
+  size_t nmatch = 1;
 
-buf.allocated = 0;
-buf.buffer = NULL;
-buf.fastmap = fastmap;
-buf.translate = 0;
-
-if ((err = re_compile_pattern(string,strlen(string),&buf)) != 0)
+ if (regcomp(&rxcache,string, REG_EXTENDED) != 0)
    {
    sprintf(OUTPUT,"Regular expression error for %s\n",string);
-   sprintf(OUTPUT,"%s\n",err);
+   CfLog(cferror,OUTPUT,"regcomp");
    return false;
    }
 
@@ -614,44 +572,52 @@ for (ip = *list; ip != NULL; ip=ip->next)
    if (EDABORTMODE && ItemMatchesRegEx(ip->name,VEDITABORT))
       {
       Verbose("Aborting search, regex %s matches line\n",VEDITABORT);
+      regfree(&rx);
       return false;
       }
-      
-   if ((match = re_match(&buf,ip->name,strlen(ip->name),0,&regs)) == strlen(ip->name))
+
+   bcopy(&rxcache,&rx,sizeof(rx)); /* To fix a bug on some implementations where rx gets emptied */
+   
+   if  (regexec(&rx,ip->name,1,&pmatch,0) == 0)
       {
-      if (ip == *list)
-         {
-         free((*list)->name);
-         if (ip->classes != NULL) 
+      if ((pmatch.rm_so == 0) && (pmatch.rm_eo == strlen(ip->name)))
+	 {
+	 if (ip == *list)
 	    {
-            free(ip->classes);
+	    free((*list)->name);
+	    if (ip->classes != NULL) 
+	       {
+	       free(ip->classes);
+	       }
+	    *list = ip->next;
+	    free((char *)ip);
+	    
+	    EditVerbose("Deleted item %s\n",string);
+	    NUMBEROFEDITS++;
+            regfree(&rx);
+	    return true;
 	    }
-         *list = ip->next;
-         free((char *)ip);
-
-         EditVerbose("Deleted item %s\n",string);
-         NUMBEROFEDITS++;
-         return true;
-         }
-      else
-         {
-         last->next = ip->next; 
-         free(ip->name);
-         if (ip->classes != NULL) 
+	 else
 	    {
-            free(ip->classes);
+	    last->next = ip->next; 
+	    free(ip->name);
+	    if (ip->classes != NULL) 
+	       {
+	       free(ip->classes);
+	       }
+	    free((char *)ip);
+	    
+	    EditVerbose("Deleted item %s\n",string);
+	    NUMBEROFEDITS++;
+	    regfree(&rx);
+	    return true;
 	    }
-         free((char *)ip);
-
-         EditVerbose("Deleted item %s\n",string);
-         NUMBEROFEDITS++;
-         return true;
-         }
-
+	 }
       }
    last = ip;
    }
 
+regfree(&rx);
 return false;
 }
 
@@ -700,7 +666,7 @@ for (ip = *list; ip != NULL; ip=ip->next)
 	    }
          free((char *)ip);
 
-         EditVerbose("Deleted item %s\n",string);
+         EditVerbose("Deleted item %s\n",ip->name);
          NUMBEROFEDITS++;
          return true;
          }
@@ -826,21 +792,14 @@ char *string,*comm,*end;
 
 { struct Item *ip;
   char buff[bufsize];
-  char fastmap[(1 << BYTEWIDTH)];
-  struct re_pattern_buffer buf;
-  struct re_registers regs;
-  int match;
-  char *err;
+  regex_t rx,rxcache;
+  regmatch_t pmatch;
+  size_t nmatch = 1;
 
-buf.allocated = 0;
-buf.buffer = NULL;
-buf.fastmap = fastmap;
-buf.translate = 0;
-
-if ((err = re_compile_pattern(string,strlen(string),&buf)) != 0)
+if (regcomp(&rxcache,string, REG_EXTENDED) != 0)
    {
    sprintf(OUTPUT,"Regular expression error for %s\n",string);
-   sprintf(OUTPUT,"%s\n",err);
+   CfLog(cferror,OUTPUT,"regcomp");
    return false;
    }
 
@@ -849,40 +808,49 @@ for (ip = *list; ip != NULL; ip=ip->next)
    if (EDABORTMODE && ItemMatchesRegEx(ip->name,VEDITABORT))
       {
       Verbose("Aborting search, regex %s matches line\n",VEDITABORT);
+      regfree(&rx);
       return false;
       }
-      
-   if ((match = re_match(&buf,ip->name,strlen(ip->name),0,&regs)) == strlen(ip->name))
+
+   bcopy(&rxcache,&rx,sizeof(rx)); /* To fix a bug on some implementations where rx gets emptied */
+   
+   if (regexec(&rx,ip->name,1,&pmatch,0) == 0)
       {
-      if (strlen(ip->name)+strlen(comm)+strlen(end)+2 > bufsize)
-         {
-         CfLog(cferror,"Bufsize overflow while commenting line - abort\n","");
-         return false;
-         }
-
-      if (strncmp(ip->name,comm,strlen(comm)) == 0) /* Already commented */
-         {
-         continue;
-         }
-
-      EditVerbose("Commenting %s%s%s\n",comm,ip->name,end);
-
-      sprintf(buff,"%s%s%s",comm,ip->name,end);
-      free(ip->name);
-
-      if ((ip->name = malloc(strlen(buff)+1)) == NULL)
-         {
-         CfLog(cferror,"malloc in CommentItemContaining\n ","");
-         FatalError("");;
-         }
-
-      strcpy(ip->name,buff);
-      NUMBEROFEDITS++;
-
-      return true;
+      if ((pmatch.rm_so == 0) && (pmatch.rm_eo == strlen(ip->name)))
+	 {
+	 if (strlen(ip->name)+strlen(comm)+strlen(end)+2 > bufsize)
+	    {
+	    CfLog(cferror,"Bufsize overflow while commenting line - abort\n","");
+            regfree(&rx);
+	    return false;
+	    }
+	 
+	 if (strncmp(ip->name,comm,strlen(comm)) == 0) /* Already commented */
+	    {
+	    continue;
+	    }
+	 
+	 EditVerbose("Commenting %s%s%s\n",comm,ip->name,end);
+	 
+	 sprintf(buff,"%s%s%s",comm,ip->name,end);
+	 free(ip->name);
+	 
+	 if ((ip->name = malloc(strlen(buff)+1)) == NULL)
+	    {
+	    CfLog(cferror,"malloc in CommentItemContaining\n ","");
+	    FatalError("");;
+	    }
+	 
+	 strcpy(ip->name,buff);
+	 NUMBEROFEDITS++;
+	 
+         regfree(&rx);
+	 return true;
+	 }
       }
    }
 
+regfree(&rx);
 return false;
 }
 
@@ -894,90 +862,90 @@ struct Item **list;
 char *string,*comm,*end;
 
 { struct Item *ip;
-  char buff[bufsize];
-  char fastmap[(1 << BYTEWIDTH)];
-  struct re_pattern_buffer buf;
-  struct re_registers regs;
-  int match;
-  char *err, *sp, *sp1, *sp2, *spc;
+  char *sp, *sp1, *sp2, *spc;
+  regex_t rx,rxcache;
+  regmatch_t pmatch;
+  size_t nmatch = 1;
 
-buf.allocated = 0;
-buf.buffer = NULL;
-buf.fastmap = fastmap;
-buf.translate = 0;
-
-if ((err = re_compile_pattern(string,strlen(string),&buf)) != 0)
+if (regcomp(&rxcache,string, REG_EXTENDED) != 0)
    {
    sprintf(OUTPUT,"Regular expression error for %s\n",string);
-   CfLog(cferror,OUTPUT,"");
-   sprintf(OUTPUT,"%s\n",err);
-   CfLog(cferror,OUTPUT,"");   
+   CfLog(cferror,OUTPUT,"regcomp");
    return false;
    }
 
 for (ip = *list; ip != NULL; ip=ip->next)
    {
-   if ((match = re_match(&buf,ip->name,strlen(ip->name),0,&regs)) == strlen(ip->name))
+   bcopy(&rxcache,&rx,sizeof(rx)); /* To fix a bug on some implementations where rx gets emptied */
+   
+   if (regexec(&rx,ip->name,1,&pmatch,0) == 0)
       {
-      if (strlen(ip->name)+strlen(comm)+strlen(end)+2 > bufsize)
-         {
-         CfLog(cferror,"Bufsize overflow while commenting line - abort\n","");
-         return false;
-         }
-
-      if (strstr(ip->name,comm) == NULL)
-         {
-         CURRENTLINEPTR = ip->next;
-         continue;
-         }
-
-      EditVerbose("Uncomment line %s\n",ip->name);
-      CURRENTLINEPTR = ip->next;
-
-      if ((sp = malloc(strlen(ip->name)+2)) == NULL)
-         {
-         CfLog(cferror,"No Memory in UnCommentNLines\n","malloc");
-         return false;
-         }
-
-      spc = sp;
-   
-      for (sp1 = ip->name; isspace(*sp1); sp1++)
-         {
-         *spc++ = *sp1;
-         }
-
-      *spc = '\0';
-
-      sp2 = ip->name+strlen(ip->name);
-
-      if ((strlen(end) != 0) && (strstr(ip->name,end) != NULL))
-         {
-         for (sp2 = ip->name+strlen(ip->name); strncmp(sp2,end,strlen(end)) != 0; sp2--)
-            {
-            }
-
-         *sp2 = '\0';
-         }
-
-      strcat(sp,sp1+strlen(comm));
-
-      if (sp2 != ip->name+strlen(ip->name))
-         {
-         strcat(sp,sp2+strlen(end));
-         }
-
-      if (strcmp(sp,ip->name) != 0)
-         {
-         NUMBEROFEDITS++;
-         }
-   
-      free(ip->name);
-      ip->name = sp;
-      return true;
+      if ((pmatch.rm_so == 0) && (pmatch.rm_eo == strlen(ip->name)))
+	 {
+	 if (strlen(ip->name)+strlen(comm)+strlen(end)+2 > bufsize)
+	    {
+	    CfLog(cferror,"Bufsize overflow while commenting line - abort\n","");
+            regfree(&rx);
+	    return false;
+	    }
+	 
+	 if (strstr(ip->name,comm) == NULL)
+	    {
+	    CURRENTLINEPTR = ip->next;
+	    continue;
+	    }
+	 
+	 EditVerbose("Uncomment line %s\n",ip->name);
+	 CURRENTLINEPTR = ip->next;
+	 
+	 if ((sp = malloc(strlen(ip->name)+2)) == NULL)
+	    {
+	    CfLog(cferror,"No Memory in UnCommentNLines\n","malloc");
+            regfree(&rx);
+	    return false;
+	    }
+	 
+	 spc = sp;
+	 
+	 for (sp1 = ip->name; isspace(*sp1); sp1++)
+	    {
+	    *spc++ = *sp1;
+	    }
+	 
+	 *spc = '\0';
+	 
+	 sp2 = ip->name+strlen(ip->name);
+	 
+	 if ((strlen(end) != 0) && (strstr(ip->name,end) != NULL))
+	    {
+	    for (sp2 = ip->name+strlen(ip->name); strncmp(sp2,end,strlen(end)) != 0; sp2--)
+	       {
+	       }
+	    
+	    *sp2 = '\0';
+	    }
+	 
+	 strcat(sp,sp1+strlen(comm));
+	 
+	 if (sp2 != ip->name+strlen(ip->name))
+	    {
+	    strcat(sp,sp2+strlen(end));
+	    }
+	 
+	 if (strcmp(sp,ip->name) != 0)
+	    {
+	    NUMBEROFEDITS++;
+	    }
+	 
+	 free(ip->name);
+	 ip->name = sp;
+         regfree(&rx);
+	 return true;
+	 }
       }
    }
 
+regfree(&rx);
 return false;
 }
 
@@ -1063,11 +1031,13 @@ struct Item **filestart;
 char *string, *comm, *end;
 
 { struct Item *ip, *ip_end = NULL;
-  char fastmap[(1 << BYTEWIDTH)];
-  struct re_pattern_buffer buf;
-  struct re_registers regs;
-  int match, linefound = false, done;
-  char *sp, *err;
+  int linefound = false, done;
+  char *sp;
+  regex_t rx;
+  regmatch_t pmatch;
+  size_t nmatch = 1;
+  
+
 
 Debug2("CommentToRegExp(list,%s %s)\n",comm,string);
 
@@ -1077,17 +1047,10 @@ if (CURRENTLINEPTR == NULL)  /* Shouldn't happen */
    return true;
    }
 
-buf.allocated = 0;
-buf.buffer = NULL;
-buf.fastmap = fastmap;
-buf.translate = 0;
-
-if ((err = re_compile_pattern(string,strlen(string),&buf)) != 0)
+if (regcomp(&rx,string, REG_EXTENDED) != 0)
    {
    sprintf(OUTPUT,"Regular expression error for %s\n",string);
-   CfLog(cferror,OUTPUT,"");
-   sprintf(OUTPUT,"%s\n",err);
-   CfLog(cferror,OUTPUT,"");   
+   CfLog(cferror,OUTPUT,"regcomp");
    return false;
    }
 
@@ -1096,21 +1059,19 @@ for (ip = CURRENTLINEPTR; (ip != NULL); ip=ip->next)
    if (EDABORTMODE && ItemMatchesRegEx(ip->name,VEDITABORT))
       {
       Verbose("Aborting search, regex %s matches line\n",VEDITABORT);
+      regfree(&rx);
       return false;
       }
       
-   if ((match = re_match(&buf,ip->name,strlen(ip->name),0,&regs)) == strlen(ip->name))
+   if (regexec(&rx,ip->name,1,&pmatch,0) == 0)
       {
-      linefound = true;
-      ip_end = ip;
-      break;
+      if ((pmatch.rm_so == 0) && (pmatch.rm_eo == strlen(ip->name)))
+	 {
+	 linefound = true;
+	 ip_end = ip;
+	 break;
+	 }
       }
-   else if (match < -1)
-      {
-      sprintf(OUTPUT,"Regular expression internal error [%s]\n",string);
-      CfLog(cferror,OUTPUT,"");
-      }
-
    }
 
 if (! linefound)
@@ -1135,6 +1096,7 @@ for (ip = CURRENTLINEPTR; ip != NULL; ip = CURRENTLINEPTR)
    if ((sp = malloc(strlen(ip->name)+strlen(comm)+strlen(end)+2)) == NULL)
       {
       CfLog(cferror,"No memory in CommentToRegExp\n","malloc");
+      regfree(&rx);
       return false;
       }
 
@@ -1151,6 +1113,7 @@ for (ip = CURRENTLINEPTR; ip != NULL; ip = CURRENTLINEPTR)
       }
    }
 
+regfree(&rx);
 return true;
 }
 
@@ -1263,37 +1226,30 @@ LineMatches (line,regexp)
 
 char *line, *regexp;
 
-{ char fastmap[(1 << BYTEWIDTH)];
-  struct re_pattern_buffer buf;
-  struct re_registers regs;
-  int match;
-  char *err;
+{ char *err;
+ regex_t rx;
+ regmatch_t pmatch;
+ size_t nmatch = 0;
 
-Debug2("LineMatches(%s,%s)\n",line,regexp);
-buf.allocated = 0;
-buf.buffer = NULL;
-buf.fastmap = fastmap;
-buf.translate = 0;
-
-if ((err = re_compile_pattern(regexp,strlen(regexp),&buf)) != 0)
+if (regcomp(&rx,regexp, REG_EXTENDED) != 0)
    {
-   sprintf(OUTPUT,"Regular expression error for %s\n",regexp);
-   sprintf(OUTPUT,"%s\n",err);
+   sprintf(OUTPUT,"Error in regular expression %s",regexp);
+   CfLog(cferror,OUTPUT,"regcomp");
    return false;
    }
 
-re_compile_fastmap (&buf);
-
-if ((match = re_match(&buf,line,strlen(line),0,&regs)) == strlen(line))
+if (regexec(&rx,line,1,&pmatch,0) == 0)
    {
-   return true;
-   }
-else if (match < -1)
-   {
-   sprintf(OUTPUT,"Regular expression internal error [%s]\n",regexp);
-   CfLog(cferror,OUTPUT,"");
+   /* Exact match of whole line */
+   
+   if ((pmatch.rm_so == 0) && (pmatch.rm_eo == strlen(line)))
+      {
+      regfree(&rx);
+      return true;
+      }
    }
 
+regfree(&rx);
 return false;
 }
 
@@ -1304,57 +1260,46 @@ GlobalReplace(liststart,search,replace)
 struct Item **liststart;
 char *search, *replace;
 
-{ int i;
+{ int i,did;
   char *sp, *start = NULL;
   struct Item *ip;
   struct Item *oldCurrentLinePtr = CURRENTLINEPTR;
-  char fastmap[(1 << BYTEWIDTH)];
-  struct re_pattern_buffer buf;
-  struct re_registers regs;
-  int match;
-  char *err;
+  regex_t rx,rxcache;
+  regmatch_t match;
+  size_t nmatch = 1;
   
 EditVerbose("Checking for replace/%s/%s\n",search,replace);
-buf.allocated = 0;
-buf.buffer = NULL;
-buf.fastmap = fastmap;
-buf.translate = 0;
 
-if ((err = re_compile_pattern(search,strlen(search),&buf)) != 0)
+if (regcomp(&rxcache,search,REG_EXTENDED) != 0)
    {
-   sprintf(OUTPUT,"Regular expression error for %s\n",search);
-   CfLog(cferror,OUTPUT,"");
-   sprintf(OUTPUT,"%s\n",err);
-   CfLog(cferror,OUTPUT,"");
+   sprintf(OUTPUT,"Error in regular expression %s",search);
+   CfLog(cferror,OUTPUT,"regcomp");
    return false;
    }
 
-re_compile_fastmap (&buf);
-
 for (ip = *liststart; ip != NULL; ip=ip->next)
    {
    if (ip->name == NULL)
       {
       continue;
       }
-   
-   if ((match = re_search(&buf,ip->name,strlen(ip->name),0,strlen(ip->name),&regs)) >= 0)
+
+   Debug("Match [%s] in [%s]\n",search,ip->name);
+   bcopy(&rxcache,&rx,sizeof(rx)); /* To fix a bug on some implementations where rx gets emptied */
+
+   if (regexec(&rx,ip->name,1,&match,0) == 0)
       {
-      start = ip->name + match;
+      start = ip->name + match.rm_so;
+      Debug("Matched string at ... %s (%d-%d)\n",start,match.rm_so,match.rm_eo);
       }
-   else if (match == -1)
+   else
       {
       continue;
       }
-   else if (match < -1)
-      {
-      sprintf(OUTPUT,"Error in regular expression [%s]!\n",search);
-      CfLog(cferror,OUTPUT,"");
-      }
 
    bzero(VBUFF,bufsize);
    
-   i = 0;
+   i = did = 0;
 
    for (sp = ip->name; *sp != '\0'; sp++)
       {
@@ -1364,29 +1309,29 @@ for (ip = *liststart; ip != NULL; ip=ip->next)
          }
       else
          {
-         sp += regs.end[0] - regs.start[0] - 1;
+         sp += match.rm_eo - match.rm_so - 1;
          VBUFF[i] = '\0';
          strcat(VBUFF,replace);
          i += strlen(replace)-1;
+	 did = true;
+
+	 bcopy(&rxcache,&rx,sizeof(rx)); /* To fix a bug on some implementations where rx gets emptied */
 	 
-         if ((match = re_search(&buf,sp,strlen(sp),0,strlen(sp),&regs)) >= 0)
+         if (regexec(&rx,sp,1,&match,0) == 0)
             {
-            start = sp + match;
+            start = sp + match.rm_so;
+	     Debug("Matched 2 string at ... %s\n",start);
             }
-         else if (match == -1)
+         else
             {
             start = 0;
             }
-          else if (match < -1)
-            {
-            sprintf(OUTPUT,"ERROR in regular expression [%s]!\n",search);
-	    CfLog(cferror,OUTPUT,"");
-            }
          }
 
       i++;
       }
 
+   Debug("Replace:\n  (%s)\nwith      (%s)\n",ip->name,VBUFF);
    CURRENTLINEPTR = ip;                  /* patch */
    InsertItemAfter(liststart,ip,VBUFF);
    CURRENTLINEPTR = ip->next;
@@ -1394,7 +1339,10 @@ for (ip = *liststart; ip != NULL; ip=ip->next)
 
    if (CURRENTLINEPTR->next)
       {
-      ip = CURRENTLINEPTR->next;
+      if (did = 0)
+	 {
+	 ip = CURRENTLINEPTR->next;
+	 }
       }
    else
       {
@@ -1404,6 +1352,7 @@ for (ip = *liststart; ip != NULL; ip=ip->next)
 
 CURRENTLINEPTR = oldCurrentLinePtr;      /* patch */
 
+regfree(&rx);
 return true;
 }
 
@@ -1620,39 +1569,29 @@ ItemMatchesRegEx(item,regex)
 
 char *item, *regex;
 
-{ char fastmap[(1 << BYTEWIDTH)];
-  struct re_pattern_buffer buf;
-  struct re_registers regs;
-  int match;
-  char *err;
+{ regex_t rx;
+  regmatch_t pmatch; 
+  size_t nmatch = 1;
 
-Debug2("ItemMatchesRegEx(%s %s)\n",item,regex);
+Debug("ItemMatchesRegEx(%s %s)\n",item,regex);
 
-buf.allocated = 0;
-buf.buffer = NULL;
-buf.fastmap = fastmap;
-buf.translate = 0;
-
-if ((err = re_compile_pattern(regex,strlen(regex),&buf)) != 0)
+if (regcomp(&rx,regex, REG_EXTENDED) != 0)
    {
    sprintf(OUTPUT,"Regular expression error for AbortAtLineMatching %s\n",regex);
-   CfLog(cferror,OUTPUT,"");
-   sprintf(OUTPUT,"%s\n",err);
-   CfLog(cferror,OUTPUT,"");
+   CfLog(cferror,OUTPUT,"regcomp");
    return true;
    }
 
-if ((match = re_match(&buf,item,strlen(item),0,&regs)) == strlen(item))
+if (regexec(&rx,item,1,&pmatch,0) == 0)
    {
-   return true;
-   }
-else if (match < -1)
-   {
-   sprintf(OUTPUT,"Regular expression internal error for AbortAtLineMatchings [%s]\n",regex);
-   CfLog(cferror,OUTPUT,"");
-   return true;  /* for safety */
+   if ((pmatch.rm_so == 0) && (pmatch.rm_eo == strlen(item)))
+      {
+      regfree(&rx);
+      return true;
+      }
    }
 
+regfree(&rx);
 return false;
 }
 
