@@ -42,7 +42,6 @@ char *file;
 
 { FILE *fp;
   struct stat statbuf;
-  int i;
 
 if (stat(file,&statbuf) == -1)
    {
@@ -73,10 +72,7 @@ if ((fp = fopen(file,"r")) == NULL)
    }
 
 
-for (i = 0; i < bufsize; i++)
-   {
-   VBUFF[i] = 0;
-   }
+bzero(VBUFF,bufsize); 
 
 while(!feof(fp))
    {
@@ -151,3 +147,71 @@ chmod(file,statbuf.st_mode);                    /* Restore file permissions etc
 chown(file,statbuf.st_uid,statbuf.st_gid);
 return true;
 }
+
+/*********************************************************************/
+
+CompareToFile(liststart,file)
+
+/* returns true if file on disk is identical to file in memory */
+
+struct Item *liststart;
+char *file;
+
+{ FILE *fp;
+  struct stat statbuf;
+  struct Item *ip = liststart;
+
+Debug("CompareToFile(%s)\n",file);
+
+if (stat(file,&statbuf) == -1)
+   {
+   return false;
+   }
+
+if (liststart == NULL)
+   {
+   return false;
+   }
+ 
+if ((fp = fopen(file,"r")) == NULL)
+   {
+   sprintf(OUTPUT,"Couldn't read file %s for editing\n",file);
+   CfLog(cferror,OUTPUT,"fopen");
+   return false;
+   }
+
+bzero(VBUFF,bufsize);
+
+for (ip = liststart; ip != NULL; ip=ip->next)
+   {
+   ReadLine(VBUFF,bufsize,fp);
+   
+   if (feof(fp) && (ip->next != NULL))
+      {
+      return false;
+      }
+
+   if ((ip->name == NULL) && (strlen(VBUFF) == 0))
+      {
+      continue;
+      }
+   if (ip->name == NULL)
+      {
+      return false;
+      }
+   if (strcmp(ip->name,VBUFF) != 0)
+      {
+      return false;
+      }
+
+   VBUFF[0] = '\0';
+   }
+
+if (!feof(fp))
+   {
+   return false;
+   }
+
+fclose(fp);
+return (true);
+}
