@@ -63,7 +63,7 @@ if ((dirh = opendir(to)) == NULL)
 
 for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
    {
-   if (!SensibleFile(dirp->d_name,to))
+   if (!SensibleFile(dirp->d_name,to,NULL))
       {
       continue;
       }
@@ -185,7 +185,7 @@ for (sp = path+strlen(path); sp != path-1; sp--)
 
          for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
             {
-            if (!SensibleFile(dirp->d_name,odir))
+            if (!SensibleFile(dirp->d_name,odir,NULL))
                {
                continue;
                } 
@@ -310,7 +310,7 @@ if (lp->next != bug_check)
 
 for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
    {
-   if (!SensibleFile(dirp->d_name,to))
+   if (!SensibleFile(dirp->d_name,to,NULL))
       {
       continue;
       }
@@ -458,6 +458,8 @@ if (IsWildItemIn(VCOPYLINKS,lastnode) || IsWildItemIn(copy,lastnode))
    ip.inclusions = NULL;
    ip.symlink = NULL;
    ip.classes = NULL;
+   ip.plus_flags = 0;
+   ip.minus_flags = 0;
    ip.server = strdup("localhost");
    Verbose("%s: Link item %s marked for copying instead\n",VPREFIX,from);
    MakeDirectoriesFor(to);
@@ -867,6 +869,8 @@ if (IsWildItemIn(VCOPYLINKS,lastnode) || IsWildItemIn(copy,lastnode))
    ip.recurse = 0;
    ip.type = 't';
    ip.backup = true;
+   ip.plus_flags = 0;
+   ip.minus_flags = 0;
    ip.exclusions = NULL;
    ip.symlink = NULL;
    Verbose("%s: Link item %s marked for copying instead\n",VPREFIX,from);
