@@ -31,6 +31,8 @@
 #include "cf.defs.h"
 #include "cf.extern.h"
 
+extern char CFLOCK[bufsize];
+
 /*****************************************************************************/
 
 CfLog(level,string,errstr)
@@ -49,6 +51,7 @@ if ((string == NULL) || (strlen(string) == 0))
    {
    case cfsilent:    if (! SILENT || VERBOSE || DEBUG || D2)
                         {
+			ShowAction();
 			printf("%s: %s",VPREFIX,string);
 			endl = true;
                         }
@@ -61,6 +64,7 @@ if ((string == NULL) || (strlen(string) == 0))
    
                      if (INFORM || VERBOSE || DEBUG || D2)
                         {
+			ShowAction();
 			printf("%s: %s",VPREFIX,string);
 			endl = true;
                         }
@@ -81,12 +85,14 @@ if ((string == NULL) || (strlen(string) == 0))
                         {
 			if ((errstr == NULL) || (strlen(errstr) > 0))
 			   {
+			   ShowAction();
 			   printf("%s: %s",VPREFIX,string);
 			   printf("%s: %s",VPREFIX,errstr);
 			   endl = true;
 			   }
 			else
 			   {
+			   ShowAction();
 			   printf("%s: %s",VPREFIX,string);
 			   endl = true;
 			   }
@@ -95,6 +101,7 @@ if ((string == NULL) || (strlen(string) == 0))
 
    case cfeditverbose: if (EDITVERBOSE || DEBUG)
                           {
+			  ShowAction();
 		   	  printf("%s: %s",VPREFIX,string);
 			  endl = true;
                           }
@@ -128,6 +135,7 @@ if ((string == NULL) || (strlen(string) == 0))
 		     
 		     if ((errstr != NULL) && (strlen(errstr) > 0))
                         {
+			ShowAction();
 		        printf("%s: %s: %s\n",VPREFIX,errstr,strerror(errno));
 			endl = true;
 			
@@ -184,3 +192,13 @@ else
    }
 }
 
+/*****************************************************************************/
+
+ShowAction()
+
+{
+if (SHOWACTIONS)
+   {
+   printf("%s:",CFLOCK);
+   }
+}
