@@ -41,7 +41,7 @@ InitHashTable()
 
 for (i = 0; i < hashtablesize; i++)
    {
-   HASH[i] = 0;
+   HASH[i] = NULL;
    }
 }
 
@@ -97,7 +97,7 @@ if ((sp = malloc(strlen(buffer)+1)) == NULL)
 strcpy(sp,buffer);
 
 slot = Hash(name);
-
+ 
 if (HASH[slot] != 0)
    {
    if (CompareMacro(name,HASH[slot]) == 0)
@@ -109,7 +109,7 @@ if (HASH[slot] != 0)
       return;
       }
 
-   while ((HASH[++slot] != 0))
+   while ((HASH[++slot % hashtablesize] != 0))
       {
       if (slot == hashtablesize-1)
          {
@@ -134,31 +134,31 @@ char *GetMacroValue(name)
 char *name;
 
 { char *sp;
-  int slot;
-
-slot = Hash(name);
+  int slot,i;
 
+i = slot = Hash(name);
+ 
 if (CompareMacro(name,HASH[slot]) != 0)
    {
    while (true)
       {
-      slot++;
+      i++;
 
-      if (CompareMacro(name,HASH[slot]) == 0)
+      if (i >= hashtablesize-1)
          {
-         for(sp = HASH[slot]; *sp != '='; sp++)
+         i = 0;
+         }
+
+      if (CompareMacro(name,HASH[i]) == 0)
+         {
+         for(sp = HASH[i]; *sp != '='; sp++)
             {
             }
 
          return(sp+1);
          }
 
-      if (slot == hashtablesize-1)
-         {
-         slot = 0;
-         }
-
-      if (slot == Hash(name))
+      if (i == slot-1)
          {
          return(getenv(name));  /* Look in environment if not found */
          }
