@@ -109,7 +109,7 @@ if ((sp = malloc(strlen(VSYSNAME.nodename)+1)) == NULL)
 
 strcpy(sp,VSYSNAME.nodename);
 SetDomainName(sp);
- 
+
 strcpy(VPREFIX,VSYSNAME.nodename);
 
 for (sp2=sp; *sp2 != '\0'; sp2++)  /* Truncate fully qualified name */
@@ -123,7 +123,7 @@ for (sp2=sp; *sp2 != '\0'; sp2++)  /* Truncate fully qualified name */
    }
 
 
-VDEFAULTBINSERVER.name = sp; 
+VDEFAULTBINSERVER.name = sp;
 
 AddClassToHeap(CanonifyName(sp));
 
@@ -337,21 +337,39 @@ int warnlevel,refresh;
   DBC *dbcp;
   db_recno_t recno;
   char dbvalue[17],dbdigest[17];
+  time_t tloc;
 	
 Debug("ChecksumChanged: key %s in db %s with data %s\n",filename,CHECKSUMDB,cfMDPrint(digest));
   
 if (CHECKSUMDB[0] == '\0')
    {
-   CfLog(cferror,"(cfd) No database defined","");
-   return false;
+   if (ISCFENGINE)
+      {
+      CfLog(cferror,"(cfd) No database defined","");
+      return false;
+      }
+   else
+      {
+      cfMDFile(filename,dbdigest);
+      for (i = 0; i < 16; i++)
+	 {
+	 if (digest[i] != dbdigest[i])
+	    {
+	    return true;
+	    }
+	 }
+      return false;
+      }
    }
 
 if (refresh)
    {
-   /* Check whether database is current */
+   /* Check whether database is current wrt local file */
 
    if (stat(filename,&stat1) == -1)
       {
+      sprintf(OUTPUT,"Couldn't stat %s\n",filename);
+      CfLog(cferror,OUTPUT,"stat");
       return false;
       }
    
@@ -362,7 +380,15 @@ if (refresh)
 	 Debug("Checksum database is older than %s...refresh needed\n",filename);
 	 needupdate = true;
 	 }
+      else
+	 {
+	 Debug("Checksum up to date..\n");
+	 }
       }
+   else
+      {
+      needupdate = true;
+      }      
    }
  
  /* Open the database. */
@@ -371,9 +397,11 @@ if ((errno = db_open(CHECKSUMDB,DB_BTREE, DB_CREATE, 0664, NULL, NULL, &dbp)) !=
    {
    sprintf(OUTPUT,"cfd: couldn't open checksum database %s\n",CHECKSUMDB);
    CfLog(cferror,OUTPUT,"db_open");
+   Debug("Ended 2\n");
    return false;
    }
- 
+
+Debug("Opening database file %s\n",CHECKSUMDB); 
 bzero(&value,sizeof(value)); 
 bzero(&key,sizeof(key));       
       
@@ -411,7 +439,7 @@ if (needupdate)
 if ((errno = dbp->get(dbp,NULL,&key,&value,0)) == 0)
    {
    /* The filename key was found in the db */
-   Debug("Comparing %16s (sent) with %s (db) ",digest,cfMDPrint(value.data));
+   Debug("Comparing %s (sent) with %s (db) ",cfMDPrint(digest),cfMDPrint(value.data));
    bcopy(value.data,dbdigest,17);
    
    for (i = 0; i < 16; i++)
@@ -480,7 +508,15 @@ else
       }
    
    dbp->close(dbp,0);
-   return false;      /* No need to warn when first installed */
+
+   if (ISCFENGINE)
+      {
+      return false;      /* No need to warn when first installed */
+      }
+   else
+      {
+      return true;
+      }
    }
 
 #else
@@ -585,4 +621,10 @@ if (gethostname(fqn, sizeof(fqn)) != -1)
       strcpy(VDOMAIN, ++ptr);
       }
    }
+
+if (strstr(VFQNAME,".") == 0)
+   {
+   strcat(VFQNAME,".");
+   strcat(VFQNAME,VDOMAIN);
+   }
 }
