@@ -48,6 +48,11 @@ mode_t *plusmask, *minusmask;
   char action = '=';
   enum modestate state = who;
 
+if (modestring == NULL)
+   {
+   return;
+   }
+
 Debug1("ParseModeString(%s)\n",modestring);
 
 gotaction = false;
