@@ -48,32 +48,40 @@ Verbose("%s: Time out\n",VPREFIX);
 RecvSocketStream(sd,buffer,toget,nothing)
  
 int sd,toget,nothing;
-char *buffer;
+char buffer[bufsize];
  
 { int already, got;
- 
+  static int fraction;
+  char *sp; 
+
+Debug("RecvSocketStream(%d)\n",toget);
+
 for (already = 0; already != toget; already += got)
    {
    got = recv(sd,buffer+already,toget-already,0);
- 
+
    if (got == -1)
       {
+      CfLog(cferror,"Couldn't recv","recv");
       return -1;
       }
  
-   if (got == 0)
-      {
-      return already;
-      }
-
-   if (strncmp(buffer,"AUTH",4) == 0 && (already == bufsize))
+   if (got == 0)   /* doesn't happen unless sock is closed */
       {
+      Debug("Transmission empty...\n");
+      fraction = 0;
       return already;
       }
 
    Debug("    (Concatenated %d from stream)\n",got);
+
+   if (strncmp(buffer,"AUTH",4) == 0 && (already == bufsize))
+      {
+      fraction = 0;
+      return already;
+      }
    }
- 
+
 return toget;
 }
  
