@@ -1128,6 +1128,8 @@ InitializeAction()                                   /* Set defaults */
  
  PLUSMASK = (mode_t)0;
  MINUSMASK = (mode_t)0;
+ PLUSFLAG = (u_long)0;
+ MINUSFLAG = (u_long)0;
  VRECURSE = 0;
  VAGE = 99999;
  VUIDNAME[0] = '*';
@@ -1145,12 +1147,13 @@ InitializeAction()                                   /* Set defaults */
  PTRAVLINKS = (short) '?';
  FORCECOPY = false;
  IMAGEBACKUP = true;
+ SECURE = false;
  ROTATE=0;
  TIDYSIZE=0;
  PROMATCHES=-1;
  PROSIGNAL=0; 
  AGETYPE='a';
- COPYTYPE = 't';
+ COPYTYPE = DEFAULTCOPYTYPE; /* 't' */
  LINKDIRS = 'k';
  USESHELL = 'y';
  LOGP = 'd';
