@@ -268,3 +268,53 @@ Verbose("(This system does not have putenv: cannot update CFALLCLASSES\n");
 
 
 #endif
+
+
+/***********************************************************/
+/* seteuid/gid() missing on some on posix systems          */
+/***********************************************************/
+
+#ifndef HAVE_SETEUID
+
+
+#ifndef const
+# define const
+#endif
+
+seteuid (uid)
+
+uid_t uid;
+
+{
+#ifdef HAVE_SETREUID
+return setreuid(-1,uid);
+#else
+Verbose("(This system does not have setreuid (patches.c)\n");
+return -1; 
+#endif 
+}
+
+#endif
+
+/***********************************************************/
+
+#ifndef HAVE_SETEGID
+
+#ifndef const
+# define const
+#endif
+
+setegid (gid)
+
+gid_t gid;
+
+{
+#ifdef HAVE_SETREGID
+return setregid(-1,gid);
+#else
+Verbose("(This system does not have setregid (patches.c)\n");
+return -1; 
+#endif 
+}
+
+#endif
