@@ -76,22 +76,18 @@ struct Process *pp;
 struct Item *procdata;
 
 { struct Item *ip, *ip2;
-  char fastmap[(1 << BYTEWIDTH)];
-  struct re_pattern_buffer buf;
-  struct re_registers regs;
   char *err, line[bufsize], *sp;
   int match, pid=-1, ret, matches=0, got;
   int BreakPipe();
   FILE *pipe;
-
+  regex_t rx,rxcache;
+  regmatch_t pmatch;
+  size_t nmatch = 1;
+  pid_t cfengine_pid = getpid();
+  
 Debug2("Looking for process %s\n",pp->expr);
 
-buf.allocated = 0;
-buf.buffer = NULL;
-buf.fastmap = fastmap;
-buf.translate = 0;
-
-if ((err = re_compile_pattern(pp->expr,strlen(pp->expr),&buf)) != 0)
+if (regcomp(&rxcache,pp->expr,REG_EXTENDED) != 0)
    {
    sprintf(OUTPUT,"Regular expression error for process pattern %s\n",pp->expr);
    CfLog(cferror,OUTPUT,"");
@@ -100,15 +96,14 @@ if ((err = re_compile_pattern(pp->expr,strlen(pp->expr),&buf)) != 0)
    return;
    }
 
-re_compile_fastmap (&buf);
-
 for (ip = procdata; ip != NULL; ip=ip->next)
    {
-   if ((match = re_search(&buf,ip->name,strlen(ip->name),0,strlen(ip->name),&regs)) >= 0)
+   bcopy(&rxcache,&rx,sizeof(rx)); /* To fix a bug on some implementations where rx gets emptied */
+   if (regexec(&rx,ip->name,1,&pmatch,0) == 0)
       {
       pid = -1;
       got = true;
-
+      
       for (ip2 = pp->inclusions; ip2 != NULL; ip2 = ip2->next)
 	 {
 	 got = false;
@@ -119,14 +114,14 @@ for (ip = procdata; ip != NULL; ip=ip->next)
 	    break;
 	    }
 	 }
-   
+      
       if (!got)
 	 {
 	 continue;
 	 }
-
+      
       got = false;
-   
+      
       for (ip2 = pp->exclusions; ip2 != NULL; ip2 = ip2->next)
 	 {
 	 if (strstr(ip->name,ip2->name))
@@ -135,7 +130,7 @@ for (ip = procdata; ip != NULL; ip=ip->next)
 	    break;
 	    }
 	 }
-
+      
       if (got)
 	 {
 	 continue;
@@ -149,7 +144,7 @@ for (ip = procdata; ip != NULL; ip=ip->next)
 	       {
 	       sp++;
 	       }
-
+	    
 	    if ((sp > ip->name) && isalnum(*(sp-1))) /* Username contains number*/
 	       {
 	       sp++;
@@ -159,9 +154,9 @@ for (ip = procdata; ip != NULL; ip=ip->next)
 	       break;
 	       }
 	    }
-      
+	 
 	 sscanf(sp,"%d",&pid);
-
+	 
 	 if (pid != -1)
 	    {
 	    break;
@@ -174,9 +169,9 @@ for (ip = procdata; ip != NULL; ip=ip->next)
 	 CfLog(cfverbose,OUTPUT,"");
 	 continue;
 	 }
-
+      
       Debug2("Found matching pid %d\n",pid);
-
+      
       matches++;
       
       if (pid == 1 && pp->signal == cfhup)
@@ -189,29 +184,33 @@ for (ip = procdata; ip != NULL; ip=ip->next)
 	 Verbose("%s: occurred while looking for %s\n",VPREFIX,pp->expr);
 	 continue;
 	 }
-
+      
       if (pp->action == 'w')
 	 {
 	 sprintf(OUTPUT,"Process warning, found: %s\n",ip->name);
 	 CfLog(cfinform,OUTPUT,"");
 	 continue;
 	 }
-
+      
       if (pp->signal != cfnosignal)
 	 {
 	 sprintf(OUTPUT,"Signalling process %d (%s) with %s\n",pid,pp->expr,SIGNALS[pp->signal]);
 	 CfLog(cfinform,OUTPUT,"");
-
+	 
 	 if (!DONTDO)
 	    {
-	    if ((ret = kill((pid_t)pid,pp->signal)) < 0)
+	    if (pid == cfengine_pid)
+	       {
+	       CfLog(cfverbose,"Cfengine will not kill itself!\n","");
+	       }
+	    else if ((ret = kill((pid_t)pid,pp->signal)) < 0)
 	       {
 	       sprintf(OUTPUT,"Couldn't send signal to pid %d\n",pid);
 	       CfLog(cfverbose,OUTPUT,"kill");
-
+	       
 	       continue;
 	       }
-
+	    
 	    if ((pp->signal == cfkill || pp->signal == cfterm) && ret >= 0)
 	       {
 	       sprintf(OUTPUT,"Killed: %s\n",ip->name);
@@ -219,22 +218,17 @@ for (ip = procdata; ip != NULL; ip=ip->next)
 	       }
 	    }
 	 }
+      }
+   }
 
-      }
-   else if (match < -1)
-      {
-      sprintf(OUTPUT,"Regular expression internal error [%s]\n",pp->expr);
-      CfLog(cferror,OUTPUT,"");
-      }
+if (matches > 0)
+   {
+   Verbose("Defining classes %s\n",pp->defines);
+   AddMultipleClasses(pp->defines);
    }
 
 if (pp->matches >= 0)
    {
-   if (matches >= 0)
-      {
-      AddMultipleClasses(pp->defines);
-      }
-
    switch (pp->comp)
       {
       case '=': if (matches != (int)pp->matches)
@@ -264,6 +258,7 @@ if (strlen(pp->restart) != 0)
    if ((matches != 0) && (pp->signal != cfkill) && (pp->signal != cfterm))
       {
       Verbose("%s: Matches found for %s - no restart sequence\n",VPREFIX,pp->expr);
+      regfree(&rx);
       return;
       }
 
@@ -272,6 +267,7 @@ if (strlen(pp->restart) != 0)
 
    if (DONTDO)
       {
+      regfree(&rx);
       return;
       }
 
@@ -281,6 +277,7 @@ if (strlen(pp->restart) != 0)
 	 {
 	 sprintf(OUTPUT,"Process restart execution failed on %s\n",pp->restart);
 	 CfLog(cferror,OUTPUT,"popen");
+	 regfree(&rx);
 	 return;
 	 }
       }
@@ -290,6 +287,7 @@ if (strlen(pp->restart) != 0)
 	 {
 	 sprintf(OUTPUT,"Process restart execution failed on %s\n",pp->restart);
 	 CfLog(cferror,OUTPUT,"popen");
+	 regfree(&rx);
 	 return;
 	 }
       }
@@ -340,5 +338,7 @@ if (strlen(pp->restart) != 0)
    sprintf(OUTPUT,"(Done with %s)\n",pp->restart);
    CfLog(cfinform,OUTPUT,"");
    }
+
+regfree(&rx); 
 }
 
