@@ -47,6 +47,7 @@ char *fqname;
   struct sockaddr_in saddr;
   struct in_addr *iaddr;           
   struct hostent *hp;
+  struct stat statbuf;
   int n, len;
 
 bzero(sendbuff,bufsize);
@@ -71,14 +72,13 @@ if (getsockname(sd,(struct sockaddr *)&saddr,&len) == -1)
    return false;
    }
 
-Verbose("Identifying as %s i.e. %s\n",inet_ntoa(saddr.sin_addr),VFQNAME);
+Debug("Identifying as %s i.e. %s, with signature %d\n",inet_ntoa(saddr.sin_addr),VFQNAME,CFSIGNATURE);
 
 iaddr = &(saddr.sin_addr);
 hp = gethostbyaddr((void *)iaddr,sizeof(struct in_addr),AF_INET);
 
-if ((hp != NULL) && (hp->h_name == NULL))
+if ((hp == NULL) || (hp->h_name == NULL))
    {
-   perror("gethostbyaddr");
    CfLog(cferror,"Couldn't lookup IP address\n","gethostbyaddr");
    return false;
    }
@@ -91,10 +91,16 @@ if (strstr(hp->h_name,VDOMAIN)== 0)
    strcat(dnsname,VDOMAIN);
    }
  
-Verbose("System name is %s and DNS name on this socket is %s\n",fqname,dnsname);
-sprintf(sendbuff,"AUTH %s %s %s",inet_ntoa(saddr.sin_addr),dnsname,getpwuid(getuid())->pw_name);
+#ifdef NT
+if (stat("/etc/passwd",&statbuf) == -1)
+   {
+   CfLog(cferror,"Make sure you have made /etc/passwd with mkpasswd for remote copy to work\n");
+   }
+#endif 
 
-if ((n = send(sd,sendbuff,bufsize,0)) != bufsize)
+sprintf(sendbuff,"CAUTH %s %s %s %d",inet_ntoa(saddr.sin_addr),dnsname,getpwuid(getuid())->pw_name,CFSIGNATURE);
+
+if ((n = send(sd,sendbuff,bufsize,0)) == -1)
    {
    printf("send\n");
    sprintf(OUTPUT,"socket verify only sent %d bytes\n",n);
