@@ -43,114 +43,7 @@
 
 #include <string.h>
 
-/*********************************************************************/
- 
-ExpandWildCardsAndDo(wildpath,buffer,function,argptr)
- 
-char *wildpath, *buffer;
-int (*function)();
-void *argptr;
 
- /* This function recursively expands a path containing wildcards */
- /* and executes the function pointed to by function for each     */
- /* matching file or directory                                    */
-
- 
-{ char *rest, extract[bufsize], construct[bufsize],varstring[bufsize], *work;
-  struct stat statbuf;
-  DIR *dirh;
-  struct dirent *dp;
-  int count, isdir = false;
-
-varstring[0] = '\0';
-
-ExpandVarstring(wildpath,varstring,NULL);
-work = varstring;
-
-Debug2("ExpandWildCardsAndDo(%s=%s)\n",wildpath,work);
- 
-extract[0] = '\0';
-
-if (*work == '/')
-   {
-   work++;
-   isdir = true;
-   }
-
-sscanf(work,"%[^/]",extract);
-rest = work + strlen(extract);
- 
-if (strlen(extract) == 0)
-   {
-   if (isdir)
-      {
-      strcat(buffer,"/");
-      }
-   (*function)(buffer,argptr);
-   return;
-   }
- 
-if (! IsWildCard(extract))
-   {
-   strcat(buffer,"/");
-   if (BufferOverflow(buffer,extract))
-       {
-       sprintf(OUTPUT,"Culprit %s\n",extract);
-       CfLog(cferror,OUTPUT,"");
-       }
-   strcat(buffer,extract);
-   ExpandWildCardsAndDo(rest,buffer,function,argptr);
-   return;
-   }
-else
-   { 
-   strcat(buffer,"/");
-   
-   if ((dirh=opendir(buffer)) == NULL)
-      {
-      sprintf(OUTPUT,"Can't open dir: %s\n",buffer);
-      CfLog(cferror,OUTPUT,"opendir");
-      return;
-      }
-
-   count = 0;
-   strcpy(construct,buffer);                 /* save relative path */
- 
-   for (dp = readdir(dirh); dp != 0; dp = readdir(dirh))
-      {
-      if (!SensibleFile(dp->d_name,buffer))
-         {
-         continue;
-         }
-
-      count++;
-      strcpy(buffer,construct);
-      strcat(buffer,dp->d_name);
-
-      if (stat(buffer,&statbuf) == -1)
-         {
-         sprintf(OUTPUT,"Can't stat %s\n\n",buffer);
-         CfLog(cferror,OUTPUT,"stat");
-         continue;
-         }
- 
-      if (S_ISDIR(statbuf.st_mode) && WildMatch(extract,dp->d_name))
-         {
-         ExpandWildCardsAndDo(rest,buffer,function,argptr);
-         }
- 
-      }
- 
-   if (count == 0)
-      {
-      sprintf(OUTPUT,"No directories matching %s in %s\n",extract,buffer);
-      CfLog(cfinform,OUTPUT,"");
-      return;
-      }
-   closedir(dirh);
-   }
-}
- 
 /*************************************************************************/
 
 IsWildCard (str)
@@ -173,7 +66,7 @@ return (strchr(str,'?') || strchr(str,'*'));
      int i, status = startofstrings;
      char lastwild = '\0';
 
-   Debug2("WildMatch(%s,%s)\n",wildptr,cmpptr);
+   Debug1("WildMatch(%s,%s)\n",wildptr,cmpptr);
      
    if (strstr(wildptr,"*") == NULL && strstr(wildptr,"?") == NULL)
       {
